@@ -40,48 +40,34 @@
 
 #include "nsXULListboxAccessible.h"
 
+#include "States.h"
+#include "nsAccessibilityService.h"
+#include "nsAccUtils.h"
+
 #include "nsIDOMXULPopupElement.h"
 #include "nsIDOMXULMultSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
-#include "nsServiceManagerUtils.h"
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsXULColumnsAccessible
 ////////////////////////////////////////////////////////////////////////////////
 
 nsXULColumnsAccessible::
-  nsXULColumnsAccessible(nsIDOMNode *aDOMNode, nsIWeakReference *aShell) :
-  nsAccessibleWrap(aDOMNode, aShell)
+  nsXULColumnsAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsAccessibleWrap(aContent, aShell)
 {
 }
 
-nsresult
-nsXULColumnsAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsXULColumnsAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_LIST;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_LIST;
 }
 
-nsresult
-nsXULColumnsAccessible::GetStateInternal(PRUint32 *aState,
-                                         PRUint32 *aExtraState)
+PRUint64
+nsXULColumnsAccessible::NativeState()
 {
-  NS_ENSURE_ARG_POINTER(aState);
-  *aState = 0;
-
-  if (IsDefunct()) {
-    if (aExtraState)
-      *aExtraState = nsIAccessibleStates::EXT_STATE_DEFUNCT;
-
-    return NS_OK_DEFUNCT_OBJECT;
-  }
-
-  *aState = nsIAccessibleStates::STATE_READONLY;
-
-  if (aExtraState)
-    *aExtraState = 0;
-
-  return NS_OK;
+  return states::READONLY;
 }
 
 
@@ -90,36 +76,21 @@ nsXULColumnsAccessible::GetStateInternal(PRUint32 *aState,
 ////////////////////////////////////////////////////////////////////////////////
 
 nsXULColumnItemAccessible::
-  nsXULColumnItemAccessible(nsIDOMNode *aDOMNode, nsIWeakReference *aShell) :
-  nsLeafAccessible(aDOMNode, aShell)
+  nsXULColumnItemAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsLeafAccessible(aContent, aShell)
 {
 }
 
-nsresult
-nsXULColumnItemAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsXULColumnItemAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_COLUMNHEADER;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_COLUMNHEADER;
 }
 
-nsresult
-nsXULColumnItemAccessible::GetStateInternal(PRUint32 *aState,
-                                            PRUint32 *aExtraState)
+PRUint64
+nsXULColumnItemAccessible::NativeState()
 {
-  NS_ENSURE_ARG_POINTER(aState);
-
-  if (IsDefunct()) {
-    if (aExtraState)
-      *aExtraState = nsIAccessibleStates::EXT_STATE_DEFUNCT;
-
-    return NS_OK_DEFUNCT_OBJECT;
-  }
-
-  *aState = nsIAccessibleStates::STATE_READONLY;
-  if (aExtraState)
-    *aExtraState = 0;
-
-  return NS_OK;
+  return states::READONLY;
 }
 
 NS_IMETHODIMP
@@ -147,7 +118,8 @@ nsXULColumnItemAccessible::DoAction(PRUint8 aIndex)
   if (aIndex != eAction_Click)
     return NS_ERROR_INVALID_ARG;
 
-  return DoCommand();
+  DoCommand();
+  return NS_OK;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -155,8 +127,8 @@ nsXULColumnItemAccessible::DoAction(PRUint8 aIndex)
 ////////////////////////////////////////////////////////////////////////////////
 
 nsXULListboxAccessible::
-  nsXULListboxAccessible(nsIDOMNode* aDOMNode, nsIWeakReference* aShell) :
-  nsXULSelectableAccessible(aDOMNode, aShell)
+  nsXULListboxAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsXULSelectableAccessible(aContent, aShell)
 {
 }
 
@@ -193,30 +165,23 @@ nsXULListboxAccessible::IsMulticolumn()
 ////////////////////////////////////////////////////////////////////////////////
 // nsXULListboxAccessible. nsIAccessible
 
-nsresult
-nsXULListboxAccessible::GetStateInternal(PRUint32 *aState,
-                                         PRUint32 *aExtraState)
+PRUint64
+nsXULListboxAccessible::NativeState()
 {
   // As a nsXULListboxAccessible we can have the following states:
-  //   STATE_FOCUSED
-  //   STATE_READONLY
-  //   STATE_FOCUSABLE
+  //   FOCUSED, READONLY, FOCUSABLE
 
   // Get focus status from base class
-  nsresult rv = nsAccessible::GetStateInternal(aState, aExtraState);
-  NS_ENSURE_A11Y_SUCCESS(rv, rv);
+  PRUint64 states = nsAccessible::NativeState();
 
-// see if we are multiple select if so set ourselves as such
-  nsCOMPtr<nsIDOMElement> element (do_QueryInterface(mDOMNode));
-  if (element) {
-    nsAutoString selType;
-    element->GetAttribute(NS_LITERAL_STRING("seltype"), selType);
-    if (!selType.IsEmpty() && selType.EqualsLiteral("multiple"))
-      *aState |= nsIAccessibleStates::STATE_MULTISELECTABLE |
-                 nsIAccessibleStates::STATE_EXTSELECTABLE;
+  // see if we are multiple select if so set ourselves as such
+
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::seltype,
+                            nsAccessibilityAtoms::multiple, eCaseMatters)) {
+      states |= states::MULTISELECTABLE | states::EXTSELECTABLE;
   }
 
-  return NS_OK;
+  return states;
 }
 
 /**
@@ -225,7 +190,7 @@ nsXULListboxAccessible::GetStateInternal(PRUint32 *aState,
 NS_IMETHODIMP nsXULListboxAccessible::GetValue(nsAString& _retval)
 {
   _retval.Truncate();
-  nsCOMPtr<nsIDOMXULSelectControlElement> select(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMXULSelectControlElement> select(do_QueryInterface(mContent));
   if (select) {
     nsCOMPtr<nsIDOMXULSelectControlItemElement> selectedItem;
     select->GetSelectedItem(getter_AddRefs(selectedItem));
@@ -235,27 +200,19 @@ NS_IMETHODIMP nsXULListboxAccessible::GetValue(nsAString& _retval)
   return NS_ERROR_FAILURE;
 }
 
-nsresult
-nsXULListboxAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsXULListboxAccessible::NativeRole()
 {
-  nsCOMPtr<nsIContent> content = do_QueryInterface(mDOMNode);
-  if (content) {
-    // A richlistbox is used with the new autocomplete URL bar,
-    // and has a parent popup <panel>
-    nsCOMPtr<nsIDOMXULPopupElement> xulPopup =
-      do_QueryInterface(content->GetParent());
-    if (xulPopup) {
-      *aRole = nsIAccessibleRole::ROLE_COMBOBOX_LIST;
-      return NS_OK;
-    }
-  }
+  // A richlistbox is used with the new autocomplete URL bar, and has a parent
+  // popup <panel>.
+  nsCOMPtr<nsIDOMXULPopupElement> xulPopup =
+    do_QueryInterface(mContent->GetParent());
+  if (xulPopup)
+    return nsIAccessibleRole::ROLE_COMBOBOX_LIST;
 
   if (IsMulticolumn())
-    *aRole = nsIAccessibleRole::ROLE_TABLE;
-  else
-    *aRole = nsIAccessibleRole::ROLE_LISTBOX;
-
-  return NS_OK;
+    return nsIAccessibleRole::ROLE_TABLE;
+  return nsIAccessibleRole::ROLE_LISTBOX;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -287,15 +244,11 @@ nsXULListboxAccessible::GetColumnCount(PRInt32 *aColumnsCout)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-
-  nsCOMPtr<nsIContent> headContent;
-  PRUint32 count = content->GetChildCount();
+  nsIContent* headContent = nsnull;
 
+  PRUint32 count = mContent->GetChildCount();
   for (PRUint32 index = 0; index < count; ++index) {
-    nsCOMPtr<nsIContent> childContent(content->GetChildAt(index));
-    NS_ENSURE_STATE(childContent);
-
+    nsIContent* childContent = mContent->GetChildAt(index);
     if (childContent->NodeInfo()->Equals(nsAccessibilityAtoms::listcols,
                                          kNameSpaceID_XUL)) {
       headContent = childContent;
@@ -306,12 +259,10 @@ nsXULListboxAccessible::GetColumnCount(PRInt32 *aColumnsCout)
     return NS_OK;
 
   PRUint32 columnCount = 0;
+
   count = headContent->GetChildCount();
-
   for (PRUint32 index = 0; index < count; ++index) {
-    nsCOMPtr<nsIContent> childContent(headContent->GetChildAt(index));
-    NS_ENSURE_STATE(childContent);
-
+    nsIContent* childContent = headContent->GetChildAt(index);
     if (childContent->NodeInfo()->Equals(nsAccessibilityAtoms::listcol,
                                          kNameSpaceID_XUL)) {
       columnCount++;
@@ -323,22 +274,22 @@ nsXULListboxAccessible::GetColumnCount(PRInt32 *aColumnsCout)
 }
 
 NS_IMETHODIMP
-nsXULListboxAccessible::GetRowCount(PRInt32 *arowCount)
+nsXULListboxAccessible::GetRowCount(PRInt32 *aRowCount)
 {
-  NS_ENSURE_ARG_POINTER(arowCount);
-  *arowCount = 0;
+  NS_ENSURE_ARG_POINTER(aRowCount);
+  *aRowCount = 0;
 
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIDOMXULSelectControlElement> element(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMXULSelectControlElement> element(do_QueryInterface(mContent));
   NS_ENSURE_STATE(element);
 
   PRUint32 itemCount = 0;
   nsresult rv = element->GetItemCount(&itemCount);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  *arowCount = itemCount;
+  *aRowCount = itemCount;
   return NS_OK;
 }
 
@@ -353,20 +304,19 @@ nsXULListboxAccessible::GetCellAt(PRInt32 aRow, PRInt32 aColumn,
     return NS_OK;
 
   nsCOMPtr<nsIDOMXULSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
 
   nsCOMPtr<nsIDOMXULSelectControlItemElement> item;
   control->GetItemAtIndex(aRow, getter_AddRefs(item));
   NS_ENSURE_TRUE(item, NS_ERROR_INVALID_ARG);
 
-  nsCOMPtr<nsIDOMNode> itemNode(do_QueryInterface(item));
+  nsCOMPtr<nsIContent> itemContent(do_QueryInterface(item));
 
-  nsCOMPtr<nsIAccessible> accessibleRow;
-  GetAccService()->GetAccessibleInWeakShell(itemNode, mWeakShell,
-                                            getter_AddRefs(accessibleRow));
-  NS_ENSURE_STATE(accessibleRow);
+  nsAccessible *row =
+    GetAccService()->GetAccessibleInWeakShell(itemContent, mWeakShell);
+  NS_ENSURE_STATE(row);
 
-  nsresult rv = accessibleRow->GetChildAt(aColumn, aAccessibleCell);
+  nsresult rv = row->GetChildAt(aColumn, aAccessibleCell);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_INVALID_ARG);
 
   return NS_OK;
@@ -421,6 +371,28 @@ nsXULListboxAccessible::GetRowIndexAt(PRInt32 aIndex, PRInt32 *aRow)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXULListboxAccessible::GetRowAndColumnIndicesAt(PRInt32 aCellIndex,
+                                                 PRInt32* aRowIndex,
+                                                 PRInt32* aColumnIndex)
+{
+  NS_ENSURE_ARG_POINTER(aRowIndex);
+  *aRowIndex = -1;
+  NS_ENSURE_ARG_POINTER(aColumnIndex);
+  *aColumnIndex = -1;
+
+  if (IsDefunct())
+    return NS_ERROR_FAILURE;
+
+  PRInt32 columnCount = 0;
+  nsresult rv = GetColumnCount(&columnCount);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *aColumnIndex = aCellIndex % columnCount;
+  *aRowIndex = aCellIndex / columnCount;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsXULListboxAccessible::GetColumnExtentAt(PRInt32 aRow, PRInt32 aColumn,
                                           PRInt32 *aCellSpans)
@@ -466,7 +438,7 @@ nsXULListboxAccessible::IsColumnSelected(PRInt32 aColumn, PRBool *aIsSelected)
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -492,7 +464,7 @@ nsXULListboxAccessible::IsRowSelected(PRInt32 aRow, PRBool *aIsSelected)
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMXULSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULSelectControlElement.");
   
@@ -517,7 +489,7 @@ nsXULListboxAccessible::GetSelectedCellCount(PRUint32* aCount)
   *aCount = 0;
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -551,7 +523,7 @@ nsXULListboxAccessible::GetSelectedColumnCount(PRUint32* aCount)
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -584,7 +556,7 @@ nsXULListboxAccessible::GetSelectedRowCount(PRUint32* aCount)
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -611,7 +583,7 @@ nsXULListboxAccessible::GetSelectedCells(nsIArray **aCells)
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -628,19 +600,16 @@ nsXULListboxAccessible::GetSelectedCells(nsIArray **aCells)
   for (; index < selectedItemsCount; index++) {
     nsCOMPtr<nsIDOMNode> itemNode;
     selectedItems->Item(index, getter_AddRefs(itemNode));
-    nsCOMPtr<nsIAccessible> item;
-    GetAccService()->GetAccessibleInWeakShell(itemNode, mWeakShell,
-                                              getter_AddRefs(item));
+    nsCOMPtr<nsIContent> itemContent(do_QueryInterface(itemNode));
+    nsAccessible *item =
+      GetAccService()->GetAccessibleInWeakShell(itemContent, mWeakShell);
 
     if (item) {
-      nsCOMPtr<nsIAccessible> cell, nextCell;
-      item->GetFirstChild(getter_AddRefs(cell));
-      while (cell) {
-        if (nsAccUtils::Role(cell) == nsIAccessibleRole::ROLE_CELL)
-          selCells->AppendElement(cell, PR_FALSE);
-
-        cell->GetNextSibling(getter_AddRefs(nextCell));
-        nextCell.swap(cell);
+      PRInt32 cellCount = item->GetChildCount();
+      for (PRInt32 cellIdx = 0; cellIdx < cellCount; cellIdx++) {
+        nsAccessible *cell = mChildren[cellIdx];
+        if (cell->Role() == nsIAccessibleRole::ROLE_CELL)
+          selCells->AppendElement(static_cast<nsIAccessible*>(cell), PR_FALSE);
       }
     }
   }
@@ -662,7 +631,7 @@ nsXULListboxAccessible::GetSelectedCellIndices(PRUint32 *aNumCells,
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -755,7 +724,7 @@ nsXULListboxAccessible::GetSelectedRowIndices(PRUint32 *aNumRows,
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
   
@@ -803,7 +772,7 @@ nsXULListboxAccessible::SelectRow(PRInt32 aRow)
     return NS_ERROR_FAILURE;
   
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -828,7 +797,7 @@ nsXULListboxAccessible::UnselectRow(PRInt32 aRow)
     return NS_ERROR_FAILURE;
   
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> control =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   NS_ASSERTION(control,
                "Doesn't implement nsIDOMXULMultiSelectControlElement.");
 
@@ -860,43 +829,45 @@ nsXULListboxAccessible::IsProbablyForLayout(PRBool *aIsProbablyForLayout)
 ////////////////////////////////////////////////////////////////////////////////
 
 nsXULListitemAccessible::
-  nsXULListitemAccessible(nsIDOMNode* aDOMNode, nsIWeakReference* aShell) :
-  nsXULMenuitemAccessible(aDOMNode, aShell)
+  nsXULListitemAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsXULMenuitemAccessible(aContent, aShell)
 {
-  mIsCheckbox = PR_FALSE;
-  nsCOMPtr<nsIDOMElement> listItem(do_QueryInterface(mDOMNode));
-  if (listItem) {
-    nsAutoString typeString;
-    nsresult res = listItem->GetAttribute(NS_LITERAL_STRING("type"), typeString);
-    if (NS_SUCCEEDED(res) && typeString.Equals(NS_LITERAL_STRING("checkbox")))
-      mIsCheckbox = PR_TRUE;
-  }
+  mIsCheckbox = mContent->AttrValueIs(kNameSpaceID_None,
+                                      nsAccessibilityAtoms::type,
+                                      nsAccessibilityAtoms::checkbox,
+                                      eCaseMatters);
 }
 
-/** Inherit the ISupports impl from nsAccessible, we handle nsIAccessibleSelectable */
 NS_IMPL_ISUPPORTS_INHERITED0(nsXULListitemAccessible, nsAccessible)
 
-already_AddRefed<nsIAccessible>
+nsAccessible *
 nsXULListitemAccessible::GetListAccessible()
 {
   if (IsDefunct())
     return nsnull;
   
   nsCOMPtr<nsIDOMXULSelectControlItemElement> listItem =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
   if (!listItem)
     return nsnull;
 
   nsCOMPtr<nsIDOMXULSelectControlElement> list;
   listItem->GetControl(getter_AddRefs(list));
 
-  nsCOMPtr<nsIDOMNode> listNode(do_QueryInterface(list));
-  if (!listNode)
+  nsCOMPtr<nsIContent> listContent(do_QueryInterface(list));
+  if (!listContent)
     return nsnull;
 
-  nsIAccessible *listAcc = nsnull;
-  GetAccService()->GetAccessibleInWeakShell(listNode, mWeakShell, &listAcc);
-  return listAcc;
+  return GetAccService()->GetAccessibleInWeakShell(listContent, mWeakShell);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// nsXULListitemAccessible nsAccessible
+
+void
+nsXULListitemAccessible::Description(nsString& aDesc)
+{
+  nsAccessibleWrap::Description(aDesc);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -909,90 +880,70 @@ nsXULListitemAccessible::GetListAccessible()
 nsresult
 nsXULListitemAccessible::GetNameInternal(nsAString& aName)
 {
-  nsCOMPtr<nsIDOMNode> child;
-  if (NS_SUCCEEDED(mDOMNode->GetFirstChild(getter_AddRefs(child)))) {
-    nsCOMPtr<nsIDOMElement> childElement (do_QueryInterface(child));
-    if (childElement) {
-      nsAutoString tagName;
-      childElement->GetLocalName(tagName);
-      if (tagName.EqualsLiteral("listcell")) {
-        childElement->GetAttribute(NS_LITERAL_STRING("label"), aName);
-        return NS_OK;
-      }
+  nsIContent* child = mContent->GetChildAt(0);
+  if (child) {
+    if (child->NodeInfo()->Equals(nsAccessibilityAtoms::listcell,
+                                  kNameSpaceID_XUL)) {
+      child->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::label, aName);
+      return NS_OK;
     }
   }
   return GetXULName(aName);
 }
 
-nsresult
-nsXULListitemAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsXULListitemAccessible::NativeRole()
 {
-  nsCOMPtr<nsIAccessible> listAcc = GetListAccessible();
-  NS_ENSURE_STATE(listAcc);
-
-  if (nsAccUtils::Role(listAcc) == nsIAccessibleRole::ROLE_TABLE) {
-    *aRole = nsIAccessibleRole::ROLE_ROW;
-    return NS_OK;
+  nsAccessible *list = GetListAccessible();
+  if (!list) {
+    NS_ERROR("No list accessible for listitem accessible!");
+    return nsIAccessibleRole::ROLE_NOTHING;
   }
 
+  if (list->Role() == nsIAccessibleRole::ROLE_TABLE)
+    return nsIAccessibleRole::ROLE_ROW;
+
   if (mIsCheckbox)
-    *aRole = nsIAccessibleRole::ROLE_CHECKBUTTON;
-  else if (nsAccUtils::Role(mParent) == nsIAccessibleRole::ROLE_COMBOBOX_LIST)
-    *aRole = nsIAccessibleRole::ROLE_COMBOBOX_OPTION;
-  else
-    *aRole = nsIAccessibleRole::ROLE_RICH_OPTION;
-  return NS_OK;
+    return nsIAccessibleRole::ROLE_CHECKBUTTON;
+
+  if (mParent && mParent->Role() == nsIAccessibleRole::ROLE_COMBOBOX_LIST)
+    return nsIAccessibleRole::ROLE_COMBOBOX_OPTION;
+
+  return nsIAccessibleRole::ROLE_RICH_OPTION;
 }
 
-nsresult
-nsXULListitemAccessible::GetStateInternal(PRUint32 *aState,
-                                          PRUint32 *aExtraState)
+PRUint64
+nsXULListitemAccessible::NativeState()
 {
-  if (mIsCheckbox) {
-    return nsXULMenuitemAccessible::GetStateInternal(aState, aExtraState);
-  }
+  if (mIsCheckbox)
+    return nsXULMenuitemAccessible::NativeState();
 
-  *aState = 0;
-
-  if (IsDefunct()) {
-    if (aExtraState)
-      *aExtraState = nsIAccessibleStates::EXT_STATE_DEFUNCT;
-
-    return NS_OK_DEFUNCT_OBJECT;
-  }
-
-  if (aExtraState)
-    *aExtraState = 0;
-
-  *aState = nsIAccessibleStates::STATE_FOCUSABLE |
-            nsIAccessibleStates::STATE_SELECTABLE;
+  PRUint64 states = states::FOCUSABLE | states::SELECTABLE;
 
   nsCOMPtr<nsIDOMXULSelectControlItemElement> listItem =
-    do_QueryInterface(mDOMNode);
+    do_QueryInterface(mContent);
 
   if (listItem) {
     PRBool isSelected;
     listItem->GetSelected(&isSelected);
     if (isSelected)
-      *aState |= nsIAccessibleStates::STATE_SELECTED;
+      states |= states::SELECTED;
+
+    if (gLastFocusedNode == mContent)
+      states |= states::FOCUSED;
 
-    if (gLastFocusedNode == mDOMNode) {
-      *aState |= nsIAccessibleStates::STATE_FOCUSED;
-    }
   }
 
-  return NS_OK;
+  return states;
 }
 
 NS_IMETHODIMP nsXULListitemAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
 {
   if (aIndex == eAction_Click && mIsCheckbox) {
     // check or uncheck
-    PRUint32 state;
-    nsresult rv = GetStateInternal(&state, nsnull);
-    NS_ENSURE_SUCCESS(rv, rv);
+    PRUint64 states = NativeState();
 
-    if (state & nsIAccessibleStates::STATE_CHECKED)
+    if (states & states::CHECKED)
       aName.AssignLiteral("uncheck");
     else
       aName.AssignLiteral("check");
@@ -1009,19 +960,12 @@ nsXULListitemAccessible::GetAllowsAnonChildAccessibles()
   return PR_TRUE;
 }
 
-nsresult
-nsXULListitemAccessible::GetAttributesInternal(nsIPersistentProperties *aAttributes)
+void
+nsXULListitemAccessible::GetPositionAndSizeInternal(PRInt32 *aPosInSet,
+                                                    PRInt32 *aSetSize)
 {
-  NS_ENSURE_ARG_POINTER(aAttributes);
-
-  // Call base class instead of nsXULMenuAccessible because menu accessible
-  // has own implementation of group attributes setting which interferes with
-  // this one.
-  nsresult rv = nsAccessible::GetAttributesInternal(aAttributes);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsAccUtils::SetAccAttrsForXULSelectControlItem(mDOMNode, aAttributes);
-  return NS_OK;
+  nsAccUtils::GetPositionAndSizeForXULSelectControlItem(mContent, aPosInSet,
+                                                        aSetSize);
 }
 
 
@@ -1030,8 +974,8 @@ nsXULListitemAccessible::GetAttributesInternal(nsIPersistentProperties *aAttribu
 ////////////////////////////////////////////////////////////////////////////////
 
 nsXULListCellAccessible::
-  nsXULListCellAccessible(nsIDOMNode* aDOMNode, nsIWeakReference* aShell):
-  nsHyperTextAccessibleWrap(aDOMNode, aShell)
+  nsXULListCellAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
 {
 }
 
@@ -1054,14 +998,12 @@ nsXULListCellAccessible::GetTable(nsIAccessibleTable **aTable)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> thisRow;
-  GetParent(getter_AddRefs(thisRow));
-  if (nsAccUtils::Role(thisRow) != nsIAccessibleRole::ROLE_ROW)
+  nsAccessible* thisRow = GetParent();
+  if (!thisRow || thisRow->Role() != nsIAccessibleRole::ROLE_ROW)
     return NS_OK;
 
-  nsCOMPtr<nsIAccessible> table;
-  thisRow->GetParent(getter_AddRefs(table));
-  if (nsAccUtils::Role(table) != nsIAccessibleRole::ROLE_TABLE)
+  nsAccessible* table = thisRow->GetParent();
+  if (!table || table->Role() != nsIAccessibleRole::ROLE_TABLE)
     return NS_OK;
 
   CallQueryInterface(table, aTable);
@@ -1077,21 +1019,21 @@ nsXULListCellAccessible::GetColumnIndex(PRInt32 *aColumnIndex)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
+  nsAccessible* row = GetParent();
+  if (!row)
+    return NS_OK;
+
   *aColumnIndex = 0;
 
-  nsCOMPtr<nsIAccessible> prevCell, tmpAcc;
-  GetPreviousSibling(getter_AddRefs(prevCell));
-
-  while (prevCell) {
-    PRUint32 role = nsAccUtils::Role(prevCell);
+  PRInt32 indexInRow = GetIndexInParent();
+  for (PRInt32 idx = 0; idx < indexInRow; idx++) {
+    nsAccessible* cell = row->GetChildAt(idx);
+    PRUint32 role = cell->Role();
     if (role == nsIAccessibleRole::ROLE_CELL ||
         role == nsIAccessibleRole::ROLE_GRID_CELL ||
         role == nsIAccessibleRole::ROLE_ROWHEADER ||
         role == nsIAccessibleRole::ROLE_COLUMNHEADER)
       (*aColumnIndex)++;
-
-    prevCell->GetPreviousSibling(getter_AddRefs(tmpAcc));
-    tmpAcc.swap(prevCell);
   }
 
   return NS_OK;
@@ -1106,15 +1048,21 @@ nsXULListCellAccessible::GetRowIndex(PRInt32 *aRowIndex)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row, prevRow;
-  GetParent(getter_AddRefs(row));
+  nsAccessible* row = GetParent();
+  if (!row)
+    return NS_OK;
 
-  while (row) {
-    if (nsAccUtils::Role(row) == nsIAccessibleRole::ROLE_ROW)
+  nsAccessible* table = row->GetParent();
+  if (!table)
+    return NS_OK;
+
+  *aRowIndex = 0;
+
+  PRInt32 indexInTable = row->GetIndexInParent();
+  for (PRInt32 idx = 0; idx < indexInTable; idx++) {
+    row = table->GetChildAt(idx);
+    if (row->Role() == nsIAccessibleRole::ROLE_ROW)
       (*aRowIndex)++;
-
-    row->GetPreviousSibling(getter_AddRefs(prevRow));
-    row.swap(prevRow);
   }
 
   return NS_OK;
@@ -1157,29 +1105,26 @@ nsXULListCellAccessible::GetColumnHeaderCells(nsIArray **aHeaderCells)
 
   nsCOMPtr<nsIAccessibleTable> table;
   GetTable(getter_AddRefs(table));
-  if (!table)
-    return NS_OK;
+  NS_ENSURE_STATE(table); // we expect to be in a listbox (table)
 
   // Get column header cell from XUL listhead.
-  nsCOMPtr<nsIAccessible> tableAcc(do_QueryInterface(table));
+  nsAccessible *list = nsnull;
 
-  nsCOMPtr<nsIAccessible> list, nextChild;
-  tableAcc->GetFirstChild(getter_AddRefs(list));
-  while (list) {
-    if (nsAccUtils::Role(list) == nsIAccessibleRole::ROLE_LIST)
+  nsRefPtr<nsAccessible> tableAcc(do_QueryObject(table));
+  PRInt32 tableChildCount = tableAcc->GetChildCount();
+  for (PRInt32 childIdx = 0; childIdx < tableChildCount; childIdx++) {
+    nsAccessible *child = tableAcc->GetChildAt(childIdx);
+    if (child->Role() == nsIAccessibleRole::ROLE_LIST) {
+      list = child;
       break;
-
-    list->GetNextSibling(getter_AddRefs(nextChild));
-    nextChild.swap(list);
+    }
   }
 
   if (list) {
     PRInt32 colIdx = -1;
     GetColumnIndex(&colIdx);
 
-    nsCOMPtr<nsIAccessible> headerCell;
-    list->GetChildAt(colIdx, getter_AddRefs(headerCell));
-
+    nsIAccessible *headerCell = list->GetChildAt(colIdx);
     if (headerCell) {
       nsresult rv = NS_OK;
       nsCOMPtr<nsIMutableArray> headerCells =
@@ -1209,8 +1154,7 @@ nsXULListCellAccessible::GetRowHeaderCells(nsIArray **aHeaderCells)
 
   nsCOMPtr<nsIAccessibleTable> table;
   GetTable(getter_AddRefs(table));
-  if (!table)
-    return NS_OK;
+  NS_ENSURE_STATE(table); // we expect to be in a listbox (table)
 
   // Calculate row header cells from ARIA markup.
   return nsAccUtils::GetHeaderCellsFor(table, this,
@@ -1229,8 +1173,7 @@ nsXULListCellAccessible::IsSelected(PRBool *aIsSelected)
 
   nsCOMPtr<nsIAccessibleTable> table;
   GetTable(getter_AddRefs(table));
-  if (!table)
-    return NS_OK;
+  NS_ENSURE_STATE(table); // we expect to be in a listbox (table)
 
   PRInt32 rowIdx = -1;
   GetRowIndex(&rowIdx);
@@ -1241,11 +1184,10 @@ nsXULListCellAccessible::IsSelected(PRBool *aIsSelected)
 ////////////////////////////////////////////////////////////////////////////////
 // nsXULListCellAccessible. nsAccessible implementation
 
-nsresult
-nsXULListCellAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsXULListCellAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_CELL;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_CELL;
 }
 
 nsresult
@@ -1259,8 +1201,7 @@ nsXULListCellAccessible::GetAttributesInternal(nsIPersistentProperties *aAttribu
   // "table-cell-index" attribute
   nsCOMPtr<nsIAccessibleTable> table;
   GetTable(getter_AddRefs(table));
-  if (!table)
-    return NS_OK;
+  NS_ENSURE_STATE(table); // we expect to be in a listbox (table)
 
   PRInt32 rowIdx = -1;
   GetRowIndex(&rowIdx);
