@@ -130,45 +130,28 @@ const APP_ICON_ATTR_NAME = "appHandlerIcon";
 //****************************************************************************//
 // Utilities
 
-function getDisplayNameForFile(aFile) {
-/*
+function getFileDisplayName(file) {
 #ifdef XP_WIN
-*/
-  if (aFile instanceof Ci.nsILocalFileWin) {
+  if (file instanceof Ci.nsILocalFileWin) {
     try {
-      return aFile.getVersionInfoField("FileDescription"); 
-    }
-    catch(ex) {
-      // fall through to the file name
-    }
+      return file.getVersionInfoField("FileDescription");
+    } catch (e) {}
   }
-/*
 #endif
 #ifdef XP_MACOSX
-*/
-  if (aFile instanceof Ci.nsILocalFileMac) {
+  if (file instanceof Ci.nsILocalFileMac) {
     try {
-      return aFile.bundleDisplayName;
-    }
-    catch(ex) {
-      // fall through to the file name
-    }
+      return file.bundleDisplayName;
+    } catch (e) {}
   }
-/*
 #endif
-*/
-
-  return Cc["@mozilla.org/network/io-service;1"].
-         getService(Ci.nsIIOService).
-         newFileURI(aFile).
-         QueryInterface(Ci.nsIURL).
-         fileName;
+  return file.leafName;
 }
 
 function getLocalHandlerApp(aFile) {
   var localHandlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"].
                         createInstance(Ci.nsILocalHandlerApp);
-  localHandlerApp.name = getDisplayNameForFile(aFile);
+  localHandlerApp.name = getFileDisplayName(aFile);
   localHandlerApp.executable = aFile;
 
   return localHandlerApp;
@@ -661,7 +644,7 @@ FeedHandlerInfo.prototype = {
     }
 
     // Add the registered web handlers.  There can be any number of these.
-    var webHandlers = this._converterSvc.getContentHandlers(this.type, {});
+    var webHandlers = this._converterSvc.getContentHandlers(this.type);
     for each (let webHandler in webHandlers)
       this._possibleApplicationHandlers.appendElement(webHandler, false);
 
@@ -686,7 +669,7 @@ FeedHandlerInfo.prototype = {
     if (defaultFeedReader) {
       let handlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"].
                        createInstance(Ci.nsIHandlerApp);
-      handlerApp.name = getDisplayNameForFile(defaultFeedReader);
+      handlerApp.name = getFileDisplayName(defaultFeedReader);
       handlerApp.QueryInterface(Ci.nsILocalHandlerApp);
       handlerApp.executable = defaultFeedReader;
 
@@ -1281,7 +1264,7 @@ var gApplicationsPane = {
         var preferredApp = aHandlerInfo.preferredApplicationHandler;
         var name;
         if (preferredApp instanceof Ci.nsILocalHandlerApp)
-          name = getDisplayNameForFile(preferredApp.executable);
+          name = getFileDisplayName(preferredApp.executable);
         else
           name = preferredApp.name;
         return this._prefsBundle.getFormattedString("useApp", [name]);
@@ -1464,7 +1447,7 @@ var gApplicationsPane = {
       menuItem.setAttribute("action", Ci.nsIHandlerInfo.useHelperApp);
       let label;
       if (possibleApp instanceof Ci.nsILocalHandlerApp)
-        label = getDisplayNameForFile(possibleApp.executable);
+        label = getFileDisplayName(possibleApp.executable);
       else
         label = possibleApp.name;
       label = this._prefsBundle.getFormattedString("useApp", [label]);
@@ -1734,9 +1717,7 @@ var gApplicationsPane = {
                       "chrome,modal,centerscreen,titlebar,dialog=yes",
                       params);
 
-    if (params.handlerApp && 
-        params.handlerApp.executable && 
-        params.handlerApp.executable.isFile()) {
+    if (this.isValidHandlerApp(params.handlerApp)) {
       handlerApp = params.handlerApp;
 
       // Add the app to the type's list of possible handlers.
@@ -1754,7 +1735,7 @@ var gApplicationsPane = {
         this._isValidHandlerExecutable(fp.file)) {
       handlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"].
                    createInstance(Ci.nsILocalHandlerApp);
-      handlerApp.name = getDisplayNameForFile(fp.file);
+      handlerApp.name = getFileDisplayName(fp.file);
       handlerApp.executable = fp.file;
 
       // Add the app to the type's list of possible handlers.
@@ -1875,7 +1856,7 @@ var gApplicationsPane = {
     // they'll only visit URLs derived from that template (i.e. with %s
     // in the template replaced by the URL of the content being handled).
 
-    if (/^https?/.test(uri.scheme))
+    if (/^https?/.test(uri.scheme) && this._prefSvc.getBoolPref("browser.chrome.favicons"))
       return uri.prePath + "/favicon.ico";
 
     return "";
