@@ -54,10 +54,8 @@
 #include "nsIDOMEvent.h"
 #include "nsIDOMNSEvent.h"
 #include "nsIDOMDragEvent.h"
-#include "nsIDOMAbstractView.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDOMDocument.h"
-#include "nsIDOMDocumentRange.h"
 #include "nsIDOMRange.h"
 #include "nsIFormControl.h"
 #include "nsIDOMHTMLAreaElement.h"
@@ -400,10 +398,9 @@ DragDataProducer::GetNodeString(nsIContent* inNode,
   // use a range to get the text-equivalent of the node
   nsCOMPtr<nsIDOMDocument> doc;
   node->GetOwnerDocument(getter_AddRefs(doc));
-  nsCOMPtr<nsIDOMDocumentRange> docRange(do_QueryInterface(doc));
-  if (docRange) {
+  if (doc) {
     nsCOMPtr<nsIDOMRange> range;
-    docRange->CreateRange(getter_AddRefs(range));
+    doc->CreateRange(getter_AddRefs(range));
     if (range) {
       range->SelectNode(node);
       range->ToString(outNodeString);
