@@ -46,15 +46,16 @@
 #include "nsICharsetAlias.h"
 #include "nsICharsetDetector.h"
 #include "nsICharsetConverterManager.h"
+#include "nsIConsoleService.h"
 #include "nsIConverterInputStream.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
-#include "nsIFile.h"
 #include "nsIFileStreams.h"
 #include "nsIInputStream.h"
 #include "nsIIPCSerializable.h"
 #include "nsIMIMEService.h"
 #include "nsIPlatformCharset.h"
+#include "nsIScriptError.h"
 #include "nsISeekableStream.h"
 #include "nsIUnicharInputStream.h"
 #include "nsIUnicodeDecoder.h"
@@ -64,6 +65,8 @@
 #include "nsFileDataProtocolHandler.h"
 #include "nsStringStream.h"
 #include "CheckedInt.h"
+#include "nsJSUtils.h"
+#include "nsPrintfCString.h"
 
 #include "plbase64.h"
 #include "prmem.h"
@@ -76,8 +79,7 @@ using namespace mozilla;
 // from NS_NewByteInputStream is held alive as long as the
 // stream is.  We do that by passing back this class instead.
 class DataOwnerAdapter : public nsIInputStream,
-                         public nsISeekableStream,
-                         public nsIIPCSerializable
+                         public nsISeekableStream
 {
   typedef nsDOMMemoryFile::DataOwner DataOwner;
 public:
@@ -92,29 +94,23 @@ public:
 
   NS_FORWARD_NSISEEKABLESTREAM(mSeekableStream->)
 
-  NS_FORWARD_NSIIPCSERIALIZABLE(mSerializableStream->)
-
 private:
   DataOwnerAdapter(DataOwner* aDataOwner,
                    nsIInputStream* aStream)
     : mDataOwner(aDataOwner), mStream(aStream),
-      mSeekableStream(do_QueryInterface(aStream)),
-      mSerializableStream(do_QueryInterface(aStream))
+      mSeekableStream(do_QueryInterface(aStream))
   {
     NS_ASSERTION(mSeekableStream, "Somebody gave us the wrong stream!");
-    NS_ASSERTION(mSerializableStream, "Somebody gave us the wrong stream!");
   }
 
   nsRefPtr<DataOwner> mDataOwner;
   nsCOMPtr<nsIInputStream> mStream;
   nsCOMPtr<nsISeekableStream> mSeekableStream;
-  nsCOMPtr<nsIIPCSerializable> mSerializableStream;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS3(DataOwnerAdapter,
+NS_IMPL_THREADSAFE_ISUPPORTS2(DataOwnerAdapter,
                               nsIInputStream,
-                              nsISeekableStream,
-                              nsIIPCSerializable)
+                              nsISeekableStream)
 
 nsresult DataOwnerAdapter::Create(DataOwner* aDataOwner,
                                   PRUint32 aStart,
@@ -146,10 +142,10 @@ DOMCI_DATA(Blob, nsDOMFile)
 NS_INTERFACE_MAP_BEGIN(nsDOMFile)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMFile)
   NS_INTERFACE_MAP_ENTRY(nsIDOMBlob)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMBlob_MOZILLA_2_0_BRANCH)
   NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsIDOMFile, mIsFullFile)
   NS_INTERFACE_MAP_ENTRY(nsIXHRSendable)
   NS_INTERFACE_MAP_ENTRY(nsICharsetDetectionObserver)
+  NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
   NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO_CONDITIONAL(File, mIsFullFile)
   NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO_CONDITIONAL(Blob, !mIsFullFile)
 NS_INTERFACE_MAP_END
@@ -171,15 +167,25 @@ DOMFileResult(nsresult rv)
   return rv;
 }
 
+/* static */ nsresult
+nsDOMFile::NewFile(nsISupports* *aNewObject)
+{
+  nsCOMPtr<nsISupports> file = do_QueryObject(new nsDOMFile(nsnull));
+  file.forget(aNewObject);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsDOMFile::GetFileName(nsAString &aFileName)
 {
+  WarnDeprecated(NS_LITERAL_CSTRING("fileName").get());
   return GetName(aFileName);
 }
 
 NS_IMETHODIMP
 nsDOMFile::GetFileSize(PRUint64 *aFileSize)
 {
+  WarnDeprecated(NS_LITERAL_CSTRING("fileSize").get());
   return GetSize(aFileSize);
 }
 
@@ -290,13 +296,6 @@ ParseSize(PRInt64 aSize, PRInt64& aStart, PRInt64& aEnd)
   }
 }
 
-NS_IMETHODIMP
-nsDOMFile::Slice(PRUint64 aStart, PRUint64 aLength,
-                 const nsAString& aContentType, nsIDOMBlob **aBlob)
-{
-  return MozSlice(aStart, aStart + aLength, aContentType, 2, aBlob);
-}
-
 NS_IMETHODIMP
 nsDOMFile::MozSlice(PRInt64 aStart, PRInt64 aEnd,
                     const nsAString& aContentType, PRUint8 optional_argc,
@@ -304,7 +303,7 @@ nsDOMFile::MozSlice(PRInt64 aStart, PRInt64 aEnd,
 {
   *aBlob = nsnull;
 
-  // Truncate aLength and aStart so that we stay within this file.
+  // Truncate aStart and aEnd so that we stay within this file.
   PRUint64 thisLength;
   nsresult rv = GetSize(&thisLength);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -321,17 +320,18 @@ nsDOMFile::MozSlice(PRInt64 aStart, PRInt64 aEnd,
   return NS_OK;
 }
 
+const PRUint32 sFileStreamFlags =
+  nsIFileInputStream::CLOSE_ON_EOF |
+  nsIFileInputStream::REOPEN_ON_REWIND |
+  nsIFileInputStream::DEFER_OPEN;
+
 NS_IMETHODIMP
 nsDOMFile::GetInternalStream(nsIInputStream **aStream)
 {
   return mIsFullFile ?
-    NS_NewLocalFileInputStream(aStream, mFile, -1, -1,
-                               nsIFileInputStream::CLOSE_ON_EOF |
-                               nsIFileInputStream::REOPEN_ON_REWIND) :
+    NS_NewLocalFileInputStream(aStream, mFile, -1, -1, sFileStreamFlags) :
     NS_NewPartialLocalFileInputStream(aStream, mFile, mStart, mLength,
-                                      -1, -1,
-                                      nsIFileInputStream::CLOSE_ON_EOF |
-                                      nsIFileInputStream::REOPEN_ON_REWIND);
+                                      -1, -1, sFileStreamFlags);
 }
 
 NS_IMETHODIMP
@@ -365,6 +365,8 @@ nsDOMFile::GetInternalUrl(nsIPrincipal* aPrincipal, nsAString& aURL)
 NS_IMETHODIMP
 nsDOMFile::GetAsText(const nsAString &aCharset, nsAString &aResult)
 {
+  WarnDeprecated(NS_LITERAL_CSTRING("getAsText").get());
+
   aResult.Truncate();
 
   nsCOMPtr<nsIInputStream> stream;
@@ -398,6 +400,8 @@ nsDOMFile::GetAsText(const nsAString &aCharset, nsAString &aResult)
 NS_IMETHODIMP
 nsDOMFile::GetAsDataURL(nsAString &aResult)
 {
+  WarnDeprecated(NS_LITERAL_CSTRING("getAsDataURL").get());
+
   aResult.AssignLiteral("data:");
 
   nsresult rv;
@@ -467,6 +471,8 @@ nsDOMFile::GetAsDataURL(nsAString &aResult)
 NS_IMETHODIMP
 nsDOMFile::GetAsBinary(nsAString &aResult)
 {
+  WarnDeprecated(NS_LITERAL_CSTRING("getAsBinary").get());
+
   aResult.Truncate();
 
   nsCOMPtr<nsIInputStream> stream;
@@ -618,6 +624,111 @@ nsDOMFile::Notify(const char* aCharset, nsDetectionConfident aConf)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMFile::Initialize(nsISupports* aOwner,
+                      JSContext* aCx,
+                      JSObject* aObj,
+                      PRUint32 aArgc,
+                      jsval* aArgv)
+{
+  nsresult rv;
+
+  if (!nsContentUtils::IsCallerChrome()) {
+    return NS_ERROR_DOM_SECURITY_ERR; // Real short trip
+  }
+
+  NS_ENSURE_TRUE(aArgc > 0, NS_ERROR_UNEXPECTED);
+
+  // We expect to get a path to represent as a File object,
+  // or an nsIFile
+  nsCOMPtr<nsIFile> file;
+  if (!JSVAL_IS_STRING(aArgv[0])) {
+    // Lets see if it's an nsIFile
+    if (!JSVAL_IS_OBJECT(aArgv[0])) {
+      return NS_ERROR_UNEXPECTED; // We're not interested
+    }
+
+    JSObject* obj = JSVAL_TO_OBJECT(aArgv[0]);
+    NS_ASSERTION(obj, "This is a bit odd");
+
+    // Is it an nsIFile
+    file = do_QueryInterface(
+      nsContentUtils::XPConnect()->
+        GetNativeOfWrapper(aCx, obj));
+    if (!file)
+      return NS_ERROR_UNEXPECTED;
+  } else {
+    // It's a string
+    JSString* str = JS_ValueToString(aCx, aArgv[0]);
+    NS_ENSURE_TRUE(str, NS_ERROR_XPC_BAD_CONVERT_JS);
+
+    nsDependentJSString xpcomStr;
+    if (!xpcomStr.init(aCx, str)) {
+      return NS_ERROR_XPC_BAD_CONVERT_JS;
+    }
+
+    nsCOMPtr<nsILocalFile> localFile;
+    rv = NS_NewLocalFile(xpcomStr, PR_FALSE, getter_AddRefs(localFile));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    file = do_QueryInterface(localFile);
+    NS_ASSERTION(file, "This should never happen");
+  }
+
+  PRBool exists;
+  rv = file->Exists(&exists);
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(exists, NS_ERROR_FILE_NOT_FOUND);
+
+  mFile = file;
+  return NS_OK;
+}
+
+/* static */ void
+nsDOMFile::WarnDeprecated(const char* aAPI)
+{
+  nsPIDOMWindow* piWindow = nsContentUtils::GetWindowFromCaller();
+
+  if (!piWindow)
+    return;
+
+  nsCOMPtr<nsIDocument> doc = do_QueryInterface(piWindow->GetExtantDocument());
+  if (!doc)
+    return;
+
+  nsIURI* uri = doc->GetDocumentURI();
+  PRUint64 windowID = doc->OuterWindowID();
+
+  nsCAutoString spec;
+  if (uri)
+    uri->GetSpec(spec);
+
+  // This is hardcoded here in English since we're past string freeze.
+  
+  nsPrintfCString warningText
+  (500,
+   "Use of File.%s is deprecated. To upgrade your code, use standard properties or use the DOM FileReader object. For more help https://developer.mozilla.org/en/DOM/FileReader",
+   aAPI);
+
+  NS_ConvertASCIItoUTF16 utf16WarningText(warningText);
+
+  nsCOMPtr<nsIScriptError2> warning =
+    do_CreateInstance(NS_SCRIPTERROR_CONTRACTID);
+  nsresult rv = warning->InitWithWindowID(utf16WarningText.get(),
+                                          NS_ConvertUTF8toUTF16(spec).get(),
+                                          nsnull, 0, 0,
+                                          nsIScriptError::warningFlag,
+                                          "DOM File API", windowID);
+  NS_ENSURE_SUCCESS(rv, /* */);
+
+  nsCOMPtr<nsIConsoleService> consoleSvc =
+    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
+
+  nsCOMPtr<nsIConsoleMessage> logWarning = do_QueryInterface(warning);
+  if (consoleSvc)
+    consoleSvc->LogMessage(logWarning);
+}
+
 nsresult
 nsDOMFile::ConvertStream(nsIInputStream *aStream,
                          const char *aCharset,
