@@ -45,7 +45,6 @@
 #include "nsIXPConnect.h"
 
 // Other includes
-#include "nsAutoLock.h"
 #include "nsAXPCNativeCallContext.h"
 #include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
@@ -59,6 +58,8 @@
 #include "nsDOMWorkerPool.h"
 #include "nsDOMWorkerXHRProxy.h"
 
+using namespace mozilla;
+
 // The list of event types that we support. This list and the defines based on
 // it determine the sizes of the listener arrays in nsDOMWorkerXHRProxy. Make
 // sure that any event types shared by both the XHR and Upload objects are
@@ -74,7 +75,8 @@ const char* const nsDOMWorkerXHREventTarget::sListenerTypes[] = {
   "progress",                          /* LISTENER_TYPE_PROGRESS */
 
   // nsIXMLHttpRequest listeners.
-  "readystatechange"                   /* LISTENER_TYPE_READYSTATECHANGE */
+  "readystatechange",                   /* LISTENER_TYPE_READYSTATECHANGE */
+  "loadend"
 };
 
 // This should always be set to the length of sListenerTypes.
@@ -236,6 +238,32 @@ nsDOMWorkerXHREventTarget::SetOnprogress(nsIDOMEventListener* aOnprogress)
   return SetOnXListener(type, aOnprogress);
 }
 
+NS_IMETHODIMP
+nsDOMWorkerXHREventTarget::GetOnloadend(nsIDOMEventListener** aOnloadend)
+{
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+  NS_ENSURE_ARG_POINTER(aOnloadend);
+
+  nsAutoString type;
+  type.AssignASCII(sListenerTypes[LISTENER_TYPE_LOADEND]);
+
+  nsCOMPtr<nsIDOMEventListener> listener = GetOnXListener(type);
+  listener.forget(aOnloadend);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMWorkerXHREventTarget::SetOnloadend(nsIDOMEventListener* aOnloadend)
+{
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+
+  nsAutoString type;
+  type.AssignASCII(sListenerTypes[LISTENER_TYPE_LOADEND]);
+
+  return SetOnXListener(type, aOnloadend);
+}
+
 nsDOMWorkerXHRUpload::nsDOMWorkerXHRUpload(nsDOMWorkerXHR* aWorkerXHR)
 : mWorkerXHR(aWorkerXHR)
 {
@@ -258,7 +286,7 @@ nsDOMWorkerXHRUpload::AddEventListener(const nsAString& aType,
                                        nsIDOMEventListener* aListener,
                                        PRBool aUseCapture)
 {
-  return AddEventListener(aType, aListener, aUseCapture, PR_FALSE, 0);
+  return AddEventListener(aType, aListener, aUseCapture, PR_FALSE, 1);
 }
 
 NS_IMETHODIMP
@@ -466,7 +494,7 @@ nsDOMWorkerXHR::Cancel()
   {
     // This lock is here to prevent a race between Cancel and GetUpload, not to
     // protect mCanceled.
-    nsAutoLock lock(mWorker->Lock());
+    MutexAutoLock lock(mWorker->GetLock());
 
     mCanceled = PR_TRUE;
     mUpload = nsnull;
@@ -612,25 +640,6 @@ nsDOMWorkerXHR::GetResponseHeader(const nsACString& aHeader,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsDOMWorkerXHR::OpenRequest(const nsACString& aMethod,
-                            const nsACString& aUrl,
-                            PRBool aAsync,
-                            const nsAString& aUser,
-                            const nsAString& aPassword)
-{
-  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
-
-  if (mCanceled) {
-    return NS_ERROR_ABORT;
-  }
-
-  nsresult rv = mXHRProxy->OpenRequest(aMethod, aUrl, aAsync, aUser, aPassword);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsDOMWorkerXHR::Open(const nsACString& aMethod, const nsACString& aUrl,
                      PRBool aAsync, const nsAString& aUser,
@@ -646,7 +655,10 @@ nsDOMWorkerXHR::Open(const nsACString& aMethod, const nsACString& aUrl,
       aAsync = PR_TRUE;
   }
 
-  return OpenRequest(aMethod, aUrl, aAsync, aUser, aPassword);
+  nsresult rv = mXHRProxy->Open(aMethod, aUrl, aAsync, aUser, aPassword);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -708,7 +720,7 @@ nsDOMWorkerXHR::SetRequestHeader(const nsACString& aHeader,
 }
 
 NS_IMETHODIMP
-nsDOMWorkerXHR::GetReadyState(PRInt32* aReadyState)
+nsDOMWorkerXHR::GetReadyState(PRUint16* aReadyState)
 {
   NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
 
@@ -818,7 +830,7 @@ nsDOMWorkerXHR::GetUpload(nsIXMLHttpRequestUpload** aUpload)
     return NS_ERROR_ABORT;
   }
 
-  nsAutoLock lock(worker->Lock());
+  MutexAutoLock lock(worker->GetLock());
 
   if (mCanceled) {
     return NS_ERROR_ABORT;
@@ -890,9 +902,21 @@ nsDOMWorkerXHR::SetWithCredentials(PRBool aWithCredentials)
   return NS_OK;
 }
 
-/* readonly attribute jsval (ArrayBuffer) mozResponseArrayBuffer; */
 NS_IMETHODIMP
-nsDOMWorkerXHR::GetMozResponseArrayBuffer(jsval *aResult)
+nsDOMWorkerXHR::GetResponseType(nsAString& aResponseText)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsDOMWorkerXHR::SetResponseType(const nsAString& aResponseText)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute jsval response; */
+NS_IMETHODIMP
+nsDOMWorkerXHR::GetResponse(JSContext *aCx, jsval *aResult)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
