@@ -493,15 +493,8 @@ nsEditingSession::SetupEditorOnWindow(nsIDOMWindow *aWindow)
   rv = editor->AddDocumentStateListener(mStateMaintainer);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // XXXbz we really shouldn't need a presShell here!
-  nsCOMPtr<nsIPresShell> presShell;
-  rv = docShell->GetPresShell(getter_AddRefs(presShell));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
-
-  nsCOMPtr<nsISelectionController> selCon = do_QueryInterface(presShell);
-  rv = editor->Init(domDoc, presShell, nsnull /* root content */,
-                    selCon, mEditorFlags);
+  rv = editor->Init(domDoc, nsnull /* root content */,
+                    nsnull, mEditorFlags);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsISelection> selection;
@@ -648,10 +641,6 @@ nsEditingSession::GetEditorForWindow(nsIDOMWindow *aWindow,
   return editorDocShell->GetEditor(outEditor);
 }
 
-#ifdef XP_MAC
-#pragma mark -
-#endif
-
 /*---------------------------------------------------------------------------
 
   OnStateChange
@@ -901,11 +890,6 @@ nsEditingSession::OnSecurityChange(nsIWebProgress *aWebProgress,
 }
 
 
-#ifdef XP_MAC
-#pragma mark -
-#endif
-
-
 /*---------------------------------------------------------------------------
 
   IsProgressForTargetDocument
@@ -1156,11 +1140,6 @@ nsEditingSession::EndPageLoad(nsIWebProgress *aWebProgress,
 #endif
 }
 
-
-#ifdef XP_MAC
-#pragma mark -
-#endif
-
 /*---------------------------------------------------------------------------
 
   GetDocShellFromWindow
