@@ -54,13 +54,11 @@ extern "C"
 }
 #endif
 
-JS_BEGIN_EXTERN_C
 #include "jsapi.h"
 #include "jsdbgapi.h"
 #ifdef LIVEWIRE
 #include "lwdbgapi.h"
 #endif
-JS_END_EXTERN_C
 
 JS_BEGIN_EXTERN_C
 
@@ -144,6 +142,16 @@ JSD_DebuggerOnForUser(JSRuntime*         jsrt,
                       JSD_UserCallbacks* callbacks,
                       void*              user);
 
+/*
+ * Startup JSD in an application that uses compartments. Debugger
+ * objects will be allocated in the same compartment as scopeobj.
+ */
+extern JSD_PUBLIC_API(JSDContext*)
+JSD_DebuggerOnForUserWithCompartment(JSRuntime*         jsrt,
+                                     JSD_UserCallbacks* callbacks,
+                                     void*              user,
+                                     JSObject*          scopeobj);
+
 /*
 * Shutdown JSD for this JSDContext
 */
@@ -242,11 +250,12 @@ JSD_ClearAllProfileData(JSDContext* jsdc);
 * If JSD_HIDE_DISABLED_FRAMES is set, this is effectively set as well.
 */
 #define JSD_MASK_TOP_FRAME_ONLY   0x20
+
 /*
-* When this flag is set, object creation will not be tracked.  This will
-* reduce the performance price you pay by enabling the debugger.
+* 0x40 was formerly used to hook into object creation.
 */
-#define JSD_DISABLE_OBJECT_TRACE  0x40
+#define JSD_DISABLE_OBJECT_TRACE_RETIRED 0x40
+
 
 extern JSD_PUBLIC_API(void)
 JSD_SetContextFlags (JSDContext* jsdc, uint32 flags);
@@ -420,10 +429,12 @@ extern JSD_PUBLIC_API(const char*)
 JSD_GetScriptFilename(JSDContext* jsdc, JSDScript *jsdscript);
 
 /*
-* Get the function name associated with this script (NULL if not a function)
+* Get the function name associated with this script (NULL if not a function).
+* If the function does not have a name the result is the string "anonymous".
+* *** new for gecko 2.0 ****
 */
-extern JSD_PUBLIC_API(const char*)
-JSD_GetScriptFunctionName(JSDContext* jsdc, JSDScript *jsdscript);
+extern JSD_PUBLIC_API(JSString *)
+JSD_GetScriptFunctionId(JSDContext* jsdc, JSDScript *jsdscript);
 
 /*
 * Get the base linenumber of the sourcefile from which this script was loaded.
@@ -652,6 +663,8 @@ JSD_DestroyAllSources( JSDContext* jsdc );
 * Add a new item for a given URL. If an iten already exists for the given URL
 * then the old item is removed.
 * 'url' may not be NULL.
+*
+* ifdef LIVEWIRE url is treated as a char* and ownership is claimed by jsd
 */
 extern JSD_PUBLIC_API(JSDSourceText*)
 JSD_NewSourceText(JSDContext* jsdc, const char* url);
@@ -793,6 +806,12 @@ JSD_SetInterruptHook(JSDContext*           jsdc,
                      JSD_ExecutionHookProc hook,
                      void*                 callerdata);
 
+/*
+* Call the interrupt hook at least once per source line
+*/
+extern JSD_PUBLIC_API(JSBool)
+JSD_EnableSingleStepInterrupts(JSDContext* jsdc, JSDScript *jsdscript, JSBool enable);
+
 /*
 * Clear the current interrupt hook.
 */
@@ -945,17 +964,10 @@ JSD_GetThisForStackFrame(JSDContext* jsdc,
 /*
 * Get the name of the function executing in this stack frame.  Especially useful
 * for native frames (without script objects.)
+* *** new for gecko 2.0 ****
 */
-extern JSD_PUBLIC_API(const char*)
-JSD_GetNameForStackFrame(JSDContext* jsdc,
-                         JSDThreadState* jsdthreadstate,
-                         JSDStackFrameInfo* jsdframe);
-
-/*
-* True if stack frame represents a native frame.
-*/
-extern JSD_PUBLIC_API(JSBool)
-JSD_IsStackFrameNative(JSDContext* jsdc,
+extern JSD_PUBLIC_API(JSString *)
+JSD_GetIdForStackFrame(JSDContext* jsdc,
                        JSDThreadState* jsdthreadstate,
                        JSDStackFrameInfo* jsdframe);
 
@@ -1270,7 +1282,7 @@ JSD_GetValueInt(JSDContext* jsdc, JSDValue* jsdval);
 * Return double value (does NOT do conversion).
 * *** new for version 1.1 ****
 */
-extern JSD_PUBLIC_API(jsdouble*)
+extern JSD_PUBLIC_API(jsdouble)
 JSD_GetValueDouble(JSDContext* jsdc, JSDValue* jsdval);
 
 /*
@@ -1285,10 +1297,18 @@ JSD_GetValueString(JSDContext* jsdc, JSDValue* jsdval);
 
 /*
 * Return name of function IFF JSDValue represents a function.
+* *** new for gecko 2.0 ****
+*/
+extern JSD_PUBLIC_API(JSString *)
+JSD_GetValueFunctionId(JSDContext* jsdc, JSDValue* jsdval);
+
+/*
+* Return function object IFF JSDValue represents a function or an object
+* wrapping a function.
 * *** new for version 1.1 ****
 */
-extern JSD_PUBLIC_API(const char*)
-JSD_GetValueFunctionName(JSDContext* jsdc, JSDValue* jsdval);
+extern JSD_PUBLIC_API(JSFunction*)
+JSD_GetValueFunction(JSDContext* jsdc, JSDValue* jsdval);
 
 /**************************************************/
 
