@@ -1,6 +1,6 @@
-/* -*- Mode: java; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- *
- * ***** BEGIN LICENSE BLOCK *****
+/* -*- Mode: java; tab-width:8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+
+/* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
  * The contents of this file are subject to the Mozilla Public License Version
@@ -13,18 +13,16 @@
  * for the specific language governing rights and limitations under the
  * License.
  *
- * The Original Code is Rhino code, released
- * May 6, 1999.
+ * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1997-2000
+ * Mozilla Foundation.
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   Igor Bukanov
- *   Ethan Hugg
- *   Milen Nankov
+ *   Michael Lipp
+ *   David P. Caldwell <inonit@inonit.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -41,52 +39,17 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("13.4.4.29 - XML prependChild()");
+START("13.4.4.32-1 - XML replace() by index, text to string");
 
-TEST(1, true, XML.prototype.hasOwnProperty("prependChild"));
+printBugNumber(291927);
 
-x =
-<alpha>
-    <bravo>
-        <charlie>one</charlie>
-    </bravo>
-</alpha>;
+var root = <root>text</root>;
 
-correct =
-<alpha>
-    <bravo>
-        <foo>bar</foo>
-        <charlie>one</charlie>
-    </bravo>
-</alpha>;
+TEST_XML(1, "text", root.child(0));
 
-x.bravo.prependChild(<foo>bar</foo>);
+root.replace(0, "new text");
 
-TEST(2, correct, x);
-
-emps =
-<employees>
-    <employee>
-        <name>John</name>
-    </employee>
-    <employee>
-        <name>Sue</name>
-    </employee>
-</employees>   
-
-correct =
-<employees>
-    <employee>
-        <prefix>Mr.</prefix>
-        <name>John</name>
-    </employee>
-    <employee>
-        <name>Sue</name>
-    </employee>
-</employees>   
-
-emps.employee.(name == "John").prependChild(<prefix>Mr.</prefix>);
-
-TEST(3, correct, emps);
+TEST_XML(2, "new text", root.child(0));
+TEST(3, <root>new text</root>, root);
 
 END();
