@@ -17,10 +17,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,46 +37,30 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var BUGNUMBER = 352846;
-var summary = 'Passing unrooted value to OBJ_DEFAULT_VALUE is GC hazard';
+var summary = 'Do not crash: __proto__ = <x/>; <x/>.lastIndexOf(this, false)';
+var BUGNUMBER = 450871;
 var actual = 'No Crash';
 var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-function prepare_xml()
+if (typeof window == 'object')
 {
-  delete XML.prototype.function::toString;
-  Object.defineProperty(Object.prototype, "toString",
-                        { get: toSource_getter, enumerable: true,
-                          configurable: true });
-  return new XML("<a>xml_text</a>");
+    actual = expect = 'Test skipped for browser based tests due destruction of the prototype';
 }
-
-var counter = 0;
-
-function toSource_getter()
+else
 {
-  // Make sure that lastInternalResult does not contain prepare_xml
-  var tmp = { toSource: function() { return [1,2,3]; } };
-  uneval(tmp);
-
-  if (counter++ < 2) return undefined;
-
-  // run GC onr overwrite the heap
-  if (typeof gc == "function") gc();
-  var x = 1e-100;
-  for (var i = 0; i != 50000; ++i)
-    var x2 = x / 4;
-
-  // Return a function that would access this in non-trivial way to
-  // check if prepare_xml() was rooted.
-  return function() { return this.toXMLString(); };
+    try
+    {
+        __proto__ = <x/>; 
+        <x/>.lastIndexOf(this, false);
+    }
+    catch(ex)
+    {
+    }
 }
 
-uneval({ toSource: prepare_xml });
-
 TEST(1, expect, actual);
 
 END();
