@@ -19,8 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov <igor@mir2.org>
- *                 Bob Clary <bob@bclary.com>
+ * Contributor(s): Andrew Schultz
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,33 +36,17 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 230216;
-var summary = 'check for numerical overflow in regexps in back reference and bounds for {} quantifier';
-var actual = '';
-var expect = '';
-var status = '';
-
-DESCRIPTION = summary;
-EXPECTED = 'error';
+var BUGNUMBER = 317714;
+var summary = 'Regression test for regression from bug 316885';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
 
-status = inSection(1) + ' check for overflow in quantifier';
+var r3="-1";
+r3[0]++;
 
-actual = 'undefined'; 
-expect = 'error';
+reportCompare(expect, actual, summary);
 
-try
-{
-  var result = eval('/a{21474836481}/.test("a")');
-  actual = 'no exception thrown';
-  status += ' result: ' + result;
-}
-catch(e)
-{
-  actual = 'error';
-}
-
-reportCompare(expect, actual, status);
 
