@@ -35,10 +35,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-349012-05.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 349012;
-var summary = 'generator recursively calling itself via close is an Error';
+var summary = 'generators with nested try finally blocks';
 var actual = '';
 var expect = '';
 
@@ -53,25 +52,42 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  var iter;
-  function gen() {
-    iter.close();
-    yield 1;
+  expect = "[object StopIteration]";
+  var expectyield   = "12";
+  var expectfinally = "Inner finally,Outer finally";
+  var actualyield = "";
+  var actualfinally = "";
+
+  function gen()
+  {
+    try {
+      try {
+        yield 1;
+      } finally {
+        actualfinally += "Inner finally";
+        yield 2;
+      }
+    } finally {
+      actualfinally += ",Outer finally";
+    }
   }
 
-  expect = /TypeError.*[aA]lready executing generator/;
+  var iter = gen();
+  actualyield += iter.next();
+  actualyield += iter.next();
   try
   {
-    iter = gen();
-    var i = iter.next();
-    print("i="+i);
+    actualyield += iter.next();
+    actual = "No exception";
   }
   catch(ex)
   {
-    print(ex + '');
     actual = ex + '';
   }
-  reportMatch(expect, actual, summary);
+ 
+  reportCompare(expect, actual, summary);
+  reportCompare(expectyield, actualyield, summary);
+  reportCompare(expectfinally, actualfinally, summary);
 
   exitFunc ('test');
 }
