@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Andreas Gal
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -52,15 +52,25 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #135 From Gary Kwong [:nth10sd]
+// ------- Comment #6 From Andreas Gal :gal
 
-  if (typeof timeout == 'function')
-  {
-    expectExitCode(6);
-    timeout(3);
-    eval("do ([]); while(y for each (x in []))");
+  function foo() {
+    var x = 4;
+    var f = (function() { return x++; });
+    var g = (function() { return x++; });
+    return [f,g];
   }
 
+  var bar = foo();
+
+  expect = '9';
+  actual = 0;
+
+  bar[0]();
+  bar[1]();
+
+  actual = String(expect);
+
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
