@@ -43,8 +43,12 @@
 
 #include "xpcprivate.h"
 #include "XPCWrapper.h"
+#include "mozilla/Mutex.h"
 #include "nsDOMJSUtils.h"
 #include "nsIScriptGlobalObject.h"
+#include "nsNullPrincipal.h"
+
+using namespace mozilla;
 
 /***************************************************************************/
 
@@ -100,18 +104,26 @@ XPCJSContextStack::Pop(JSContext * *_retval)
     if(idx > 0)
     {
         --idx; // Advance to new top of the stack
+
         XPCJSContextInfo & e = mStack[idx];
         NS_ASSERTION(!e.frame || e.cx, "Shouldn't have frame without a cx!");
-        if(e.cx && e.frame)
+        NS_ASSERTION(!e.suspendDepth || e.cx, "Shouldn't have suspendDepth without a cx!");
+        if(e.cx)
         {
-            JS_RestoreFrameChain(e.cx, e.frame);
-            e.frame = nsnull;
+            if(e.suspendDepth)
+            {
+                JS_ResumeRequest(e.cx, e.suspendDepth);
+                e.suspendDepth = 0;
+            }
+
+            if(e.frame)
+            {
+                // Pop() can be called outside any request for e.cx.
+                JSAutoRequest ar(e.cx);
+                JS_RestoreFrameChain(e.cx, e.frame);
+                e.frame = nsnull;
+            }
         }
-
-        if(e.requestDepth)
-            JS_ResumeRequest(e.cx, e.requestDepth);
-
-        e.requestDepth = 0;
     }
     return NS_OK;
 }
@@ -119,12 +131,11 @@ XPCJSContextStack::Pop(JSContext * *_retval)
 static nsIPrincipal*
 GetPrincipalFromCx(JSContext *cx)
 {
-    nsIScriptContext* scriptContext = GetScriptContextFromJSContext(cx);
-    if (scriptContext)
+    nsIScriptContextPrincipal* scp = GetScriptContextPrincipalFromJSContext(cx);
+    if(scp)
     {
-        nsCOMPtr<nsIScriptObjectPrincipal> globalData =
-            do_QueryInterface(scriptContext->GetGlobalObject());
-        if (globalData)
+        nsIScriptObjectPrincipal* globalData = scp->GetObjectPrincipal();
+        if(globalData)
             return globalData->GetPrincipal();
     }
     return nsnull;
@@ -134,6 +145,7 @@ GetPrincipalFromCx(JSContext *cx)
 NS_IMETHODIMP
 XPCJSContextStack::Push(JSContext * cx)
 {
+    JS_ASSERT_IF(cx, JS_GetContextThread(cx));
     if(!mStack.AppendElement(cx))
         return NS_ERROR_OUT_OF_MEMORY;
     if(mStack.Length() > 1)
@@ -141,19 +153,19 @@ XPCJSContextStack::Push(JSContext * cx)
         XPCJSContextInfo & e = mStack[mStack.Length() - 2];
         if(e.cx)
         {
-            if (e.cx == cx)
+            if(e.cx == cx)
             {
                 nsIScriptSecurityManager* ssm = XPCWrapper::GetSecurityManager();
-                if (ssm)
+                if(ssm)
                 {
                     nsIPrincipal* globalObjectPrincipal =
                         GetPrincipalFromCx(cx);
-                    if (globalObjectPrincipal)
+                    if(globalObjectPrincipal)
                     {
                         nsIPrincipal* subjectPrincipal = ssm->GetCxSubjectPrincipal(cx);
                         PRBool equals = PR_FALSE;
                         globalObjectPrincipal->Equals(subjectPrincipal, &equals);
-                        if (equals)
+                        if(equals)
                         {
                             return NS_OK;
                         }
@@ -161,10 +173,14 @@ XPCJSContextStack::Push(JSContext * cx)
                 }
             }
 
-            e.frame = JS_SaveFrameChain(e.cx);
+            {
+                // Push() can be called outside any request for e.cx.
+                JSAutoRequest ar(e.cx);
+                e.frame = JS_SaveFrameChain(e.cx);
+            }
 
-            if(e.cx != cx && JS_GetContextThread(e.cx))
-                e.requestDepth = JS_SuspendRequest(e.cx);
+            if(!cx)
+                e.suspendDepth = JS_SuspendRequest(e.cx);
         }
     }
     return NS_OK;
@@ -182,7 +198,7 @@ XPCJSContextStack::DEBUG_StackHasJSContext(JSContext*  aJSContext)
 #endif
 
 static JSBool
-SafeGlobalResolve(JSContext *cx, JSObject *obj, jsval id)
+SafeGlobalResolve(JSContext *cx, JSObject *obj, jsid id)
 {
     JSBool resolved;
     return JS_ResolveStandardClass(cx, obj, id, &resolved);
@@ -191,21 +207,15 @@ SafeGlobalResolve(JSContext *cx, JSObject *obj, jsval id)
 static void
 SafeFinalize(JSContext* cx, JSObject* obj)
 {
-#ifndef XPCONNECT_STANDALONE
     nsIScriptObjectPrincipal* sop =
         static_cast<nsIScriptObjectPrincipal*>(xpc_GetJSPrivate(obj));
     NS_IF_RELEASE(sop);
-#endif
 }
 
 static JSClass global_class = {
     "global_for_XPCJSContextStack_SafeJSContext",
-#ifndef XPCONNECT_STANDALONE
     JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS | JSCLASS_GLOBAL_FLAGS,
-#else
-    0,
-#endif
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, SafeGlobalResolve, JS_ConvertStub, SafeFinalize,
     JSCLASS_NO_OPTIONAL_MEMBERS
 };
@@ -216,22 +226,21 @@ XPCJSContextStack::GetSafeJSContext(JSContext * *aSafeJSContext)
 {
     if(!mSafeJSContext)
     {
-#ifndef XPCONNECT_STANDALONE
         // Start by getting the principal holder and principal for this
         // context.  If we can't manage that, don't bother with the rest.
-        nsCOMPtr<nsIPrincipal> principal =
-            do_CreateInstance("@mozilla.org/nullprincipal;1");
+        nsRefPtr<nsNullPrincipal> principal = new nsNullPrincipal();
         nsCOMPtr<nsIScriptObjectPrincipal> sop;
         if(principal)
         {
-            sop = new PrincipalHolder(principal);
+            nsresult rv = principal->Init();
+            if(NS_SUCCEEDED(rv))
+              sop = new PrincipalHolder(principal);
         }
         if(!sop)
         {
             *aSafeJSContext = nsnull;
             return NS_ERROR_FAILURE;
         }        
-#endif /* !XPCONNECT_STANDALONE */
         
         JSRuntime *rt;
         XPCJSRuntime* xpcrt;
@@ -241,17 +250,29 @@ XPCJSContextStack::GetSafeJSContext(JSContext * *aSafeJSContext)
 
         if(xpc && (xpcrt = xpc->GetRuntime()) && (rt = xpcrt->GetJSRuntime()))
         {
+            JSObject *glob;
             mSafeJSContext = JS_NewContext(rt, 8192);
             if(mSafeJSContext)
             {
                 // scoped JS Request
-                AutoJSRequestWithNoCallContext req(mSafeJSContext);
-                JSObject *glob;
-                glob = JS_NewObject(mSafeJSContext, &global_class, NULL, NULL);
+                JSAutoRequest req(mSafeJSContext);
+
+                // Because we can run off the main thread, we create an MT
+                // global object. Our principal is the unique key.
+                JSCompartment *compartment;
+                nsresult rv = xpc_CreateMTGlobalObject(mSafeJSContext,
+                                                       &global_class,
+                                                       principal, &glob,
+                                                       &compartment);
+                if(NS_FAILED(rv))
+                    glob = nsnull;
 
-#ifndef XPCONNECT_STANDALONE
                 if(glob)
                 {
+                    // Make sure the context is associated with a proper compartment
+                    // and not the default compartment.
+                    JS_SetGlobalObject(mSafeJSContext, glob);
+
                     // Note: make sure to set the private before calling
                     // InitClasses
                     nsIScriptObjectPrincipal* priv = nsnull;
@@ -268,24 +289,26 @@ XPCJSContextStack::GetSafeJSContext(JSContext * *aSafeJSContext)
                 // nsIScriptObjectPrincipal ownership is either handled by the
                 // nsCOMPtr or dealt with, or we'll release in the finalize
                 // hook.
-#endif
-                if(!glob || NS_FAILED(xpc->InitClasses(mSafeJSContext, glob)))
+                if(glob && NS_FAILED(xpc->InitClasses(mSafeJSContext, glob)))
                 {
-                    // Explicitly end the request since we are about to kill
-                    // the JSContext that 'req' will try to use when it
-                    // goes out of scope.
-                    req.EndRequest();
-                    JS_DestroyContext(mSafeJSContext);
-                    mSafeJSContext = nsnull;
+                    glob = nsnull;
                 }
-                // Save it off so we can destroy it later, even if
-                // mSafeJSContext has been set to another context
-                // via SetSafeJSContext.  If we don't get here,
-                // then mSafeJSContext must have been set via
-                // SetSafeJSContext, and we're not responsible for
-                // destroying the passed-in context.
-                mOwnSafeJSContext = mSafeJSContext;
+
             }
+            if(mSafeJSContext && !glob)
+            {
+                // Destroy the context outside the scope of JSAutoRequest that
+                // uses the context in its destructor.
+                JS_DestroyContext(mSafeJSContext);
+                mSafeJSContext = nsnull;
+            }
+            // Save it off so we can destroy it later, even if
+            // mSafeJSContext has been set to another context
+            // via SetSafeJSContext.  If we don't get here,
+            // then mSafeJSContext must have been set via
+            // SetSafeJSContext, and we're not responsible for
+            // destroying the passed-in context.
+            mOwnSafeJSContext = mSafeJSContext;
         }
     }
 
@@ -311,43 +334,21 @@ XPCJSContextStack::SetSafeJSContext(JSContext * aSafeJSContext)
 /***************************************************************************/
 
 PRUintn           XPCPerThreadData::gTLSIndex       = BAD_TLS_INDEX;
-PRLock*           XPCPerThreadData::gLock           = nsnull;
+Mutex*            XPCPerThreadData::gLock           = nsnull;
 XPCPerThreadData* XPCPerThreadData::gThreads        = nsnull;
 XPCPerThreadData *XPCPerThreadData::sMainThreadData = nsnull;
 void *            XPCPerThreadData::sMainJSThread   = nsnull;
 
-static jsuword
-GetThreadStackLimit()
-{
-    int stackDummy;
-    jsuword stackLimit, currentStackAddr = (jsuword)&stackDummy;
-
-    const jsuword kStackSize = 0x80000;   // 512k
-
-#if JS_STACK_GROWTH_DIRECTION < 0
-    stackLimit = (currentStackAddr > kStackSize)
-                 ? currentStackAddr - kStackSize
-                 : 0;
-#else
-    stackLimit = (currentStackAddr + kStackSize > currentStackAddr)
-                 ? currentStackAddr + kStackSize
-                 : (jsuword) -1;
-#endif
-
-  return stackLimit;
-}
-
 XPCPerThreadData::XPCPerThreadData()
     :   mJSContextStack(new XPCJSContextStack()),
         mNextThread(nsnull),
         mCallContext(nsnull),
-        mResolveName(0),
+        mResolveName(JSID_VOID),
         mResolvingWrapper(nsnull),
         mExceptionManager(nsnull),
         mException(nsnull),
         mExceptionManagerNotAvailable(JS_FALSE),
-        mAutoRoots(nsnull),
-        mStackLimit(GetThreadStackLimit())
+        mAutoRoots(nsnull)
 #ifdef XPC_CHECK_WRAPPER_THREADSAFETY
       , mWrappedNativeThreadsafetyReportDepth(0)
 #endif
@@ -355,7 +356,7 @@ XPCPerThreadData::XPCPerThreadData()
     MOZ_COUNT_CTOR(xpcPerThreadData);
     if(gLock)
     {
-        nsAutoLock lock(gLock);
+        MutexAutoLock lock(*gLock);
         mNextThread = gThreads;
         gThreads = this;
     }
@@ -377,6 +378,11 @@ XPCPerThreadData::Cleanup()
 
 XPCPerThreadData::~XPCPerThreadData()
 {
+    /* Be careful to ensure that both any update to |gThreads| and the
+       decision about whether or not to destroy the lock, are done
+       atomically.  See bug 557586. */
+    PRBool doDestroyLock = PR_FALSE;
+
     MOZ_COUNT_DTOR(xpcPerThreadData);
 
     Cleanup();
@@ -384,7 +390,7 @@ XPCPerThreadData::~XPCPerThreadData()
     // Unlink 'this' from the list of threads.
     if(gLock)
     {
-        nsAutoLock lock(gLock);
+        MutexAutoLock lock(*gLock);
         if(gThreads == this)
             gThreads = mNextThread;
         else
@@ -400,11 +406,13 @@ XPCPerThreadData::~XPCPerThreadData()
                 cur = cur->mNextThread;
             }
         }
+        if (!gThreads)
+            doDestroyLock = PR_TRUE;
     }
 
-    if(gLock && !gThreads)
+    if(gLock && doDestroyLock)
     {
-        PR_DestroyLock(gLock);
+        delete gLock;
         gLock = nsnull;
     }
 }
@@ -450,21 +458,19 @@ XPCPerThreadData::GetDataImpl(JSContext *cx)
 
     if(!gLock)
     {
-        gLock = PR_NewLock();
-        if(!gLock)
-            return nsnull;
+        gLock = new Mutex("XPCPerThreadData.gLock");
     }
 
     if(gTLSIndex == BAD_TLS_INDEX)
     {
-        nsAutoLock lock(gLock);
+        MutexAutoLock lock(*gLock);
         // check again now that we have the lock...
         if(gTLSIndex == BAD_TLS_INDEX)
         {
             if(PR_FAILURE ==
                PR_NewThreadPrivateIndex(&gTLSIndex, xpc_ThreadDataDtorCB))
             {
-                NS_ASSERTION(0, "PR_NewThreadPrivateIndex failed!");
+                NS_ERROR("PR_NewThreadPrivateIndex failed!");
                 gTLSIndex = BAD_TLS_INDEX;
                 return nsnull;
             }
@@ -477,14 +483,14 @@ XPCPerThreadData::GetDataImpl(JSContext *cx)
         data = new XPCPerThreadData();
         if(!data || !data->IsValid())
         {
-            NS_ASSERTION(0, "new XPCPerThreadData() failed!");
+            NS_ERROR("new XPCPerThreadData() failed!");
             if(data)
                 delete data;
             return nsnull;
         }
         if(PR_FAILURE == PR_SetThreadPrivate(gTLSIndex, data))
         {
-            NS_ASSERTION(0, "PR_SetThreadPrivate failed!");
+            NS_ERROR("PR_SetThreadPrivate failed!");
             delete data;
             return nsnull;
         }
@@ -492,7 +498,7 @@ XPCPerThreadData::GetDataImpl(JSContext *cx)
 
     if(cx && !sMainJSThread && NS_IsMainThread())
     {
-        sMainJSThread = cx->thread;
+        sMainJSThread = cx->thread();
 
         sMainThreadData = data;
 
@@ -518,7 +524,7 @@ XPCPerThreadData::CleanupAllThreads()
 
     if(gLock)
     {
-        nsAutoLock lock(gLock);
+        MutexAutoLock lock(*gLock);
 
         for(XPCPerThreadData* cur = gThreads; cur; cur = cur->mNextThread)
             count++;
