@@ -43,7 +43,6 @@
 #ifndef nsCSSRules_h_
 #define nsCSSRules_h_
 
-#include "Rule.h"
 #include "mozilla/css/GroupRule.h"
 #include "nsIDOMCSSMediaRule.h"
 #include "nsIDOMCSSMozDocumentRule.h"
@@ -90,13 +89,12 @@ public:
   virtual void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
 #endif
 
-  // nsICSSRule methods
+  // Rule methods
   virtual void SetStyleSheet(nsCSSStyleSheet* aSheet); //override GroupRule
   virtual PRInt32 GetType() const;
-  virtual already_AddRefed<nsICSSRule> Clone() const;
-  virtual nsIDOMCSSRule* GetDOMRuleWeak(nsresult *aResult)
+  virtual already_AddRefed<Rule> Clone() const;
+  virtual nsIDOMCSSRule* GetDOMRule()
   {
-    *aResult = NS_OK;
     return this;
   }
 
@@ -134,12 +132,11 @@ public:
   virtual void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
 #endif
 
-  // nsICSSRule methods
+  // Rule methods
   virtual PRInt32 GetType() const;
-  virtual already_AddRefed<nsICSSRule> Clone() const;
-  virtual nsIDOMCSSRule* GetDOMRuleWeak(nsresult *aResult)
+  virtual already_AddRefed<Rule> Clone() const;
+  virtual nsIDOMCSSRule* GetDOMRule()
   {
-    *aResult = NS_OK;
     return this;
   }
 
@@ -156,7 +153,8 @@ public:
   enum Function {
     eURL,
     eURLPrefix,
-    eDomain
+    eDomain,
+    eRegExp
   };
 
   struct URL {
@@ -233,11 +231,11 @@ public:
   virtual void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
 #endif
 
-  // nsICSSRule methods
+  // Rule methods
   DECL_STYLE_RULE_INHERIT
 
   virtual PRInt32 GetType() const;
-  virtual already_AddRefed<nsICSSRule> Clone() const;
+  virtual already_AddRefed<mozilla::css::Rule> Clone() const;
 
   // nsIDOMCSSRule interface
   NS_DECL_NSIDOMCSSRULE
@@ -297,9 +295,9 @@ public:
   virtual void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
 #endif
 
-  // nsICSSRule methods
+  // Rule methods
   virtual PRInt32 GetType() const;
-  virtual already_AddRefed<nsICSSRule> Clone() const;
+  virtual already_AddRefed<Rule> Clone() const;
 
   // nsIDOMCSSRule interface
   NS_DECL_NSIDOMCSSRULE
@@ -329,10 +327,7 @@ public:
   void DropReference() { mRule = nsnull; }
   virtual mozilla::css::Declaration* GetCSSDeclaration(PRBool aAllocate);
   virtual nsresult SetCSSDeclaration(mozilla::css::Declaration* aDecl);
-  virtual nsresult GetCSSParsingEnvironment(nsIURI** aSheetURI,
-                                            nsIURI** aBaseURI,
-                                            nsIPrincipal** aSheetPrincipal,
-                                            mozilla::css::Loader** aCSSLoader);
+  virtual void GetCSSParsingEnvironment(CSSParsingEnvironment& aCSSParseEnv);
   virtual nsIDocument* DocToUpdate();
 
   NS_IMETHOD_(nsrefcnt) AddRef();
@@ -363,21 +358,21 @@ public:
   {
     mKeys.SwapElements(aKeys);
   }
-
+private:
   nsCSSKeyframeRule(const nsCSSKeyframeRule& aCopy);
   ~nsCSSKeyframeRule();
-
-  NS_DECL_ISUPPORTS
+public:
+  NS_DECL_ISUPPORTS_INHERITED
 
   // nsIStyleRule methods
 #ifdef DEBUG
   virtual void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
 #endif
 
-  // nsICSSRule methods
+  // Rule methods
   DECL_STYLE_RULE_INHERIT
   virtual PRInt32 GetType() const;
-  virtual already_AddRefed<nsICSSRule> Clone() const;
+  virtual already_AddRefed<mozilla::css::Rule> Clone() const;
 
   // nsIDOMCSSRule interface
   NS_DECL_NSIDOMCSSRULE
@@ -405,9 +400,10 @@ public:
     : mName(aName)
   {
   }
+private:
   nsCSSKeyframesRule(const nsCSSKeyframesRule& aCopy);
   ~nsCSSKeyframesRule();
-
+public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIStyleRule methods
@@ -415,12 +411,11 @@ public:
   virtual void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
 #endif
 
-  // nsICSSRule methods
+  // Rule methods
   virtual PRInt32 GetType() const;
-  virtual already_AddRefed<nsICSSRule> Clone() const;
-  virtual nsIDOMCSSRule* GetDOMRuleWeak(nsresult *aResult)
+  virtual already_AddRefed<mozilla::css::Rule> Clone() const;
+  virtual nsIDOMCSSRule* GetDOMRule()
   {
-    *aResult = NS_OK;
     return this;
   }
 
