@@ -27,7 +27,7 @@ function sync_httpd_setup() {
 
   let catapultEngine = Engines.get("catapult");
   let engines        = {catapult: {version: catapultEngine.version,
-                                   syncID:  catapultEngine.syncID}}
+                                   syncID:  catapultEngine.syncID}};
 
   // Track these using the collections helper, which keeps modified times
   // up-to-date.
@@ -42,7 +42,7 @@ function sync_httpd_setup() {
     "/1.1/johndoe/storage/meta/global": upd("meta",    globalWBO.handler()),
     "/1.1/johndoe/storage/clients":     upd("clients", clientsColl.handler()),
     "/1.1/johndoe/storage/crypto/keys": upd("crypto",  keysWBO.handler())
-  }
+  };
   return httpd_setup(handlers);
 }
 
@@ -55,16 +55,17 @@ function setUp() {
 }
 
 function generateAndUploadKeys() {
-  CollectionKeys.generateNewKeys();
+  generateNewKeys();
   let serverKeys = CollectionKeys.asWBO("crypto", "keys");
   serverKeys.encrypt(Weave.Service.syncKeyBundle);
   return serverKeys.upload("http://localhost:8080/1.1/johndoe/storage/crypto/keys").success;
 }
 
-function test_backoff500(next) {
+
+add_test(function test_backoff500() {
   _("Test: HTTP 500 sets backoff status.");
-  let server = sync_httpd_setup();
   setUp();
+  let server = sync_httpd_setup();
 
   let engine = Engines.get("catapult");
   engine.enabled = true;
@@ -83,13 +84,13 @@ function test_backoff500(next) {
     Status.resetBackoff();
     Service.startOver();
   }
-  server.stop(next);
-}
+  server.stop(run_next_test);
+});
 
-function test_backoff503(next) {
+add_test(function test_backoff503() {
   _("Test: HTTP 503 with Retry-After header leads to backoff notification and sets backoff status.");
-  let server = sync_httpd_setup();
   setUp();
+  let server = sync_httpd_setup();
 
   const BACKOFF = 42;
   let engine = Engines.get("catapult");
@@ -116,13 +117,13 @@ function test_backoff503(next) {
     Status.resetBackoff();
     Service.startOver();
   }
-  server.stop(next);
-}
+  server.stop(run_next_test);
+});
 
-function test_overQuota(next) {
+add_test(function test_overQuota() {
   _("Test: HTTP 400 with body error code 14 means over quota.");
-  let server = sync_httpd_setup();
   setUp();
+  let server = sync_httpd_setup();
 
   let engine = Engines.get("catapult");
   engine.enabled = true;
@@ -142,10 +143,10 @@ function test_overQuota(next) {
     Status.resetSync();
     Service.startOver();
   }
-  server.stop(next);
-}
+  server.stop(run_next_test);
+});
 
-function test_service_networkError(next) {
+add_test(function test_service_networkError() {
   _("Test: Connection refused error from Service.sync() leads to the right status code.");
   setUp();
   // Provoke connection refused.
@@ -164,13 +165,13 @@ function test_service_networkError(next) {
     Status.resetSync();
     Service.startOver();
   }
-  next();
-}
+  run_next_test();
+});
 
-function test_service_offline(next) {
+add_test(function test_service_offline() {
   _("Test: Wanting to sync in offline mode leads to the right status code but does not increment the ignorable error count.");
   setUp();
-  Svc.IO.offline = true;
+  Services.io.offline = true;
   Service._ignorableErrorCount = 0;
 
   try {
@@ -185,14 +186,14 @@ function test_service_offline(next) {
     Status.resetSync();
     Service.startOver();
   }
-  Svc.IO.offline = false;
-  next();
-}
+  Services.io.offline = false;
+  run_next_test();
+});
 
-function test_service_reset_ignorableErrorCount(next) {
+add_test(function test_service_reset_ignorableErrorCount() {
   _("Test: Successful sync resets the ignorable error count.");
-  let server = sync_httpd_setup();
   setUp();
+  let server = sync_httpd_setup();
   Service._ignorableErrorCount = 10;
 
   // Disable the engine so that sync completes.
@@ -213,13 +214,13 @@ function test_service_reset_ignorableErrorCount(next) {
     Status.resetSync();
     Service.startOver();
   }
-  server.stop(next);
-}
+  server.stop(run_next_test);
+});
 
-function test_engine_networkError(next) {
+add_test(function test_engine_networkError() {
   _("Test: Network related exceptions from engine.sync() lead to the right status code.");
-  let server = sync_httpd_setup();
   setUp();
+  let server = sync_httpd_setup();
   Service._ignorableErrorCount = 0;
 
   let engine = Engines.get("catapult");
@@ -241,14 +242,43 @@ function test_engine_networkError(next) {
     Status.resetSync();
     Service.startOver();
   }
-  server.stop(next);
-}
+  server.stop(run_next_test);
+});
+
+add_test(function test_resource_timeout() {
+  setUp();
+  let server = sync_httpd_setup();
+
+  let engine = Engines.get("catapult");
+  engine.enabled = true;
+  // Resource throws this when it encounters a timeout.
+  engine.exception = Components.Exception("Aborting due to channel inactivity.",
+                                          Cr.NS_ERROR_NET_TIMEOUT);
+
+  try {
+    do_check_eq(Status.sync, SYNC_SUCCEEDED);
+
+    do_check_true(generateAndUploadKeys());
+
+    Service.login();
+    Service.sync();
+
+    do_check_eq(Status.sync, LOGIN_FAILED_NETWORK_ERROR);
+  } finally {
+    Status.resetSync();
+    Service.startOver();
+  }
+  server.stop(run_next_test);
+});
+
 
 // Slightly misplaced test as it doesn't actually test checkServerError,
 // but the observer for "weave:engine:sync:apply-failed".
-function test_engine_applyFailed(next) {
-  let server = sync_httpd_setup();
+// This test should be the last one since it monkeypatches the engine object
+// and we should only have one engine object throughout the file (bug 629664).
+add_test(function test_engine_applyFailed() {
   setUp();
+  let server = sync_httpd_setup();
 
   let engine = Engines.get("catapult");
   engine.enabled = true;
@@ -270,24 +300,14 @@ function test_engine_applyFailed(next) {
     Status.resetSync();
     Service.startOver();
   }
-  server.stop(next);
-}
+  server.stop(run_next_test);
+});
+
 
 function run_test() {
   if (DISABLE_TESTS_BUG_604565)
     return;
 
-  do_test_pending();
-
-  // Register engine once.
   Engines.register(CatapultEngine);
-  asyncChainTests(test_backoff500,
-                  test_backoff503,
-                  test_overQuota,
-                  test_service_networkError,
-                  test_service_offline,
-                  test_service_reset_ignorableErrorCount,
-                  test_engine_networkError,
-                  test_engine_applyFailed,
-                  do_test_finished)();
+  run_next_test();
 }
