@@ -24,6 +24,8 @@
 #   Fredrik Holmqvist <thesuckiestemail@yahoo.se>
 #   Asaf Romano <mozilla.mano@sent.com>
 #   Ehsan Akhgari <ehsan.akhgari@gmail.com>
+#   Kathleen Brade <brade@pearlcrescent.com>
+#   Mark Smith <mcs@pearlcrescent.com>
 #
 # Alternatively, the contents of this file may be used under the terms of
 # either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -39,6 +41,23 @@
 #
 # ***** END LICENSE BLOCK *****
 
+var ContentAreaUtils = {
+  get ioService() {
+    delete this.ioService;
+    return this.ioService =
+      Components.classes["@mozilla.org/network/io-service;1"]
+                .getService(Components.interfaces.nsIIOService);
+  },
+
+  get stringBundle() {
+    delete this.stringBundle;
+    return this.stringBundle =
+      Components.classes["@mozilla.org/intl/stringbundle;1"]
+                .getService(Components.interfaces.nsIStringBundleService)
+                .createBundle("chrome://global/locale/contentAreaCommands.properties");
+  }
+}
+
 /**
  * urlSecurityCheck: JavaScript wrapper for checkLoadURIWithPrincipal
  * and checkLoadURIStrWithPrincipal.
@@ -69,8 +88,13 @@ function urlSecurityCheck(aURL, aPrincipal, aFlags)
     else
       secMan.checkLoadURIStrWithPrincipal(aPrincipal, aURL, aFlags);
   } catch (e) {
-    // XXXmano: dump the principal url here too
-    throw "Load of " + aURL + " denied.";
+    let principalStr = "";
+    try {
+      principalStr = " from " + aPrincipal.URI.spec;
+    }
+    catch(e2) { }
+
+    throw "Load of " + aURL + principalStr + " denied.";
   }
 }
 
@@ -86,7 +110,7 @@ function isContentFrame(aFocusedWindow)
 }
 
 
-// Clientelle: (Make sure you don't break any of these)
+// Clientele: (Make sure you don't break any of these)
 //  - File    ->  Save Page/Frame As...
 //  - Context ->  Save Page/Frame As...
 //  - Context ->  Save Link As...
@@ -108,12 +132,12 @@ function saveURL(aURL, aFileName, aFilePickerTitleKey, aShouldBypassCache,
                  aSkipPrompt, aReferrer)
 {
   internalSave(aURL, null, aFileName, null, null, aShouldBypassCache,
-               aFilePickerTitleKey, null, aReferrer, aSkipPrompt);
+               aFilePickerTitleKey, null, aReferrer, aSkipPrompt, null);
 }
 
 // Just like saveURL, but will get some info off the image before
 // calling internalSave
-// Clientelle: (Make sure you don't break any of these)
+// Clientele: (Make sure you don't break any of these)
 //  - Context ->  Save Image As...
 const imgICache = Components.interfaces.imgICache;
 const nsISupportsCString = Components.interfaces.nsISupportsCString;
@@ -139,14 +163,8 @@ function saveImageURL(aURL, aFileName, aFilePickerTitleKey, aShouldBypassCache,
     }
   }
   internalSave(aURL, null, aFileName, contentDisposition, contentType,
-               aShouldBypassCache, aFilePickerTitleKey, null, aReferrer, aSkipPrompt);
-}
-
-function saveFrameDocument()
-{
-  var focusedWindow = document.commandDispatcher.focusedWindow;
-  if (isContentFrame(focusedWindow))
-    saveDocument(focusedWindow.document);
+               aShouldBypassCache, aFilePickerTitleKey, null, aReferrer,
+               aSkipPrompt, null);
 }
 
 function saveDocument(aDocument, aSkipPrompt)
@@ -155,20 +173,32 @@ function saveDocument(aDocument, aSkipPrompt)
     throw "Must have a document when calling saveDocument";
 
   // We want to use cached data because the document is currently visible.
+  var ifreq =
+    aDocument.defaultView
+             .QueryInterface(Components.interfaces.nsIInterfaceRequestor);
+
   var contentDisposition = null;
   try {
     contentDisposition =
-      aDocument.defaultView
-               .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-               .getInterface(Components.interfaces.nsIDOMWindowUtils)
-               .getDocumentMetadata("content-disposition");
+      ifreq.getInterface(Components.interfaces.nsIDOMWindowUtils)
+           .getDocumentMetadata("content-disposition");
   } catch (ex) {
     // Failure to get a content-disposition is ok
   }
+
+  var cacheKey = null;
+  try {
+    cacheKey =
+      ifreq.getInterface(Components.interfaces.nsIWebNavigation)
+           .QueryInterface(Components.interfaces.nsIWebPageDescriptor);
+  } catch (ex) {
+    // We might not find it in the cache.  Oh, well.
+  }
+
   internalSave(aDocument.location.href, aDocument, null, contentDisposition,
                aDocument.contentType, false, null, null,
                aDocument.referrer ? makeURI(aDocument.referrer) : null,
-               aSkipPrompt);
+               aSkipPrompt, cacheKey);
 }
 
 function DownloadListener(win, transfer) {
@@ -218,14 +248,29 @@ const kSaveAsType_Complete = 0; // Save document with attached objects.
 const kSaveAsType_Text     = 2; // Save document, converting to plain text.
 
 /**
- * internalSave: Used when saving a document or URL. This method:
- *  - Determines a local target filename to use (unless parameter
- *    aChosenData is non-null)
- *  - Determines content-type if possible
+ * internalSave: Used when saving a document or URL.
+ *
+ * If aChosenData is null, this method:
+ *  - Determines a local target filename to use
  *  - Prompts the user to confirm the destination filename and save mode
- *    (content-type affects this)
+ *    (aContentType affects this)
+ *  - [Note] This process involves the parameters aURL, aReferrer (to determine
+ *    how aURL was encoded), aDocument, aDefaultFileName, aFilePickerTitleKey,
+ *    and aSkipPrompt.
+ *
+ * If aChosenData is non-null, this method:
+ *  - Uses the provided source URI and save file name
+ *  - Saves the document as complete DOM if possible (aDocument present and
+ *    right aContentType)
+ *  - [Note] The parameters aURL, aDefaultFileName, aFilePickerTitleKey, and
+ *    aSkipPrompt are ignored.
+ *
+ * In any case, this method:
  *  - Creates a 'Persist' object (which will perform the saving in the
  *    background) and then starts it.
+ *  - [Note] This part of the process only involves the parameters aDocument,
+ *    aShouldBypassCache and aReferrer. The source, the save name and the save
+ *    mode are the ones determined previously.
  *
  * @param aURL
  *        The String representation of the URL of the document being saved
@@ -252,42 +297,48 @@ const kSaveAsType_Text     = 2; // Save document, converting to plain text.
  * @param aSkipPrompt [optional]
  *        If set to true, we will attempt to save the file to the
  *        default downloads folder without prompting.
+ * @param aCacheKey [optional]
+ *        If set will be passed to saveURI.  See nsIWebBrowserPersist for
+ *        allowed values.
  */
 function internalSave(aURL, aDocument, aDefaultFileName, aContentDisposition,
                       aContentType, aShouldBypassCache, aFilePickerTitleKey,
-                      aChosenData, aReferrer, aSkipPrompt)
+                      aChosenData, aReferrer, aSkipPrompt, aCacheKey)
 {
   if (aSkipPrompt == undefined)
     aSkipPrompt = false;
 
-  // Note: aDocument == null when this code is used by save-link-as...
-  var saveMode = GetSaveModeForContentType(aContentType);
-  var isDocument = aDocument != null && saveMode != SAVEMODE_FILEONLY;
-  var saveAsType = kSaveAsType_Complete;
+  if (aCacheKey == undefined)
+    aCacheKey = null;
 
-  var file, fileURL;
+  // Note: aDocument == null when this code is used by save-link-as...
+  var saveMode = GetSaveModeForContentType(aContentType, aDocument);
+
+  var file, sourceURI, saveAsType;
   // Find the URI object for aURL and the FileName/Extension to use when saving.
   // FileName/Extension will be ignored if aChosenData supplied.
-  var fileInfo = new FileInfo(aDefaultFileName);
-  if (aChosenData)
+  if (aChosenData) {
     file = aChosenData.file;
-  else {
+    sourceURI = aChosenData.uri;
+    saveAsType = kSaveAsType_Complete;
+  } else {
     var charset = null;
     if (aDocument)
       charset = aDocument.characterSet;
     else if (aReferrer)
       charset = aReferrer.originCharset;
+    var fileInfo = new FileInfo(aDefaultFileName);
     initFileInfo(fileInfo, aURL, charset, aDocument,
                  aContentType, aContentDisposition);
+    sourceURI = fileInfo.uri;
+
     var fpParams = {
       fpTitleKey: aFilePickerTitleKey,
-      isDocument: isDocument,
       fileInfo: fileInfo,
       contentType: aContentType,
       saveMode: saveMode,
-      saveAsType: saveAsType,
-      file: file,
-      fileURL: fileURL
+      saveAsType: kSaveAsType_Complete,
+      file: file
     };
 
     if (!getTargetFile(fpParams, aSkipPrompt))
@@ -296,40 +347,70 @@ function internalSave(aURL, aDocument, aDefaultFileName, aContentDisposition,
       return;
 
     saveAsType = fpParams.saveAsType;
-    saveMode = fpParams.saveMode;
     file = fpParams.file;
-    fileURL = fpParams.fileURL;
   }
 
-  if (!fileURL)
-    fileURL = makeFileURI(file);
-
   // XXX We depend on the following holding true in appendFiltersForContentType():
   // If we should save as a complete page, the saveAsType is kSaveAsType_Complete.
   // If we should save as text, the saveAsType is kSaveAsType_Text.
-  var useSaveDocument = isDocument &&
+  var useSaveDocument = aDocument &&
                         (((saveMode & SAVEMODE_COMPLETE_DOM) && (saveAsType == kSaveAsType_Complete)) ||
                          ((saveMode & SAVEMODE_COMPLETE_TEXT) && (saveAsType == kSaveAsType_Text)));
   // If we're saving a document, and are saving either in complete mode or
   // as converted text, pass the document to the web browser persist component.
   // If we're just saving the HTML (second option in the list), send only the URI.
-  var source = useSaveDocument ? aDocument : fileInfo.uri;
   var persistArgs = {
-    source      : source,
-    contentType : (!aChosenData && useSaveDocument &&
-                   saveAsType == kSaveAsType_Text) ?
-                  "text/plain" : null,
-    target      : fileURL,
-    postData    : isDocument ? getPostData(aDocument) : null,
-    bypassCache : aShouldBypassCache
+    sourceURI         : sourceURI,
+    sourceReferrer    : aReferrer,
+    sourceDocument    : useSaveDocument ? aDocument : null,
+    targetContentType : (saveAsType == kSaveAsType_Text) ? "text/plain" : null,
+    targetFile        : file,
+    sourceCacheKey    : aCacheKey,
+    sourcePostData    : aDocument ? getPostData(aDocument) : null,
+    bypassCache       : aShouldBypassCache
   };
 
+  // Start the actual save process
+  internalPersist(persistArgs);
+}
+
+/**
+ * internalPersist: Creates a 'Persist' object (which will perform the saving
+ *  in the background) and then starts it.
+ *
+ * @param persistArgs.sourceURI
+ *        The nsIURI of the document being saved
+ * @param persistArgs.sourceCacheKey [optional]
+ *        If set will be passed to saveURI
+ * @param persistArgs.sourceDocument [optional]
+ *        The document to be saved, or null if not saving a complete document
+ * @param persistArgs.sourceReferrer
+ *        Required and used only when persistArgs.sourceDocument is NOT present,
+ *        the nsIURI of the referrer to use, or null if no referrer should be
+ *        sent.
+ * @param persistArgs.sourcePostData
+ *        Required and used only when persistArgs.sourceDocument is NOT present,
+ *        represents the POST data to be sent along with the HTTP request, and
+ *        must be null if no POST data should be sent.
+ * @param persistArgs.targetFile
+ *        The nsIFile of the file to create
+ * @param persistArgs.targetContentType
+ *        Required and used only when persistArgs.sourceDocument is present,
+ *        determines the final content type of the saved file, or null to use
+ *        the same content type as the source document. Currently only
+ *        "text/plain" is meaningful.
+ * @param persistArgs.bypassCache
+ *        If true, the document will always be refetched from the server
+ */
+function internalPersist(persistArgs)
+{
   var persist = makeWebBrowserPersist();
 
   // Calculate persist flags.
   const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
-  const flags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
-  if (aShouldBypassCache)
+  const flags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES |
+                nsIWBP.PERSIST_FLAGS_FORCE_ALLOW_COOKIES;
+  if (persistArgs.bypassCache)
     persist.persistFlags = flags | nsIWBP.PERSIST_FLAGS_BYPASS_CACHE;
   else
     persist.persistFlags = flags | nsIWBP.PERSIST_FLAGS_FROM_CACHE;
@@ -337,26 +418,32 @@ function internalSave(aURL, aDocument, aDefaultFileName, aContentDisposition,
   // Leave it to WebBrowserPersist to discover the encoding type (or lack thereof):
   persist.persistFlags |= nsIWBP.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;
 
+  // Find the URI associated with the target file
+  var targetFileURL = makeFileURI(persistArgs.targetFile);
+
   // Create download and initiate it (below)
   var tr = Components.classes["@mozilla.org/transfer;1"].createInstance(Components.interfaces.nsITransfer);
+  tr.init(persistArgs.sourceURI,
+          targetFileURL, "", null, null, null, persist);
+  persist.progressListener = new DownloadListener(window, tr);
 
-  if (useSaveDocument) {
+  if (persistArgs.sourceDocument) {
     // Saving a Document, not a URI:
     var filesFolder = null;
-    if (persistArgs.contentType != "text/plain") {
+    if (persistArgs.targetContentType != "text/plain") {
       // Create the local directory into which to save associated files.
-      filesFolder = file.clone();
+      filesFolder = persistArgs.targetFile.clone();
 
       var nameWithoutExtension = getFileBaseName(filesFolder.leafName);
-      var filesFolderLeafName = getStringBundle().formatStringFromName("filesFolder",
-                                                                       [nameWithoutExtension],
-                                                                       1);
+      var filesFolderLeafName =
+        ContentAreaUtils.stringBundle
+                        .formatStringFromName("filesFolder", [nameWithoutExtension], 1);
 
       filesFolder.leafName = filesFolderLeafName;
     }
 
     var encodingFlags = 0;
-    if (persistArgs.contentType == "text/plain") {
+    if (persistArgs.targetContentType == "text/plain") {
       encodingFlags |= nsIWBP.ENCODE_FLAGS_FORMATTED;
       encodingFlags |= nsIWBP.ENCODE_FLAGS_ABSOLUTE_LINKS;
       encodingFlags |= nsIWBP.ENCODE_FLAGS_NOFRAMES_CONTENT;
@@ -366,18 +453,12 @@ function internalSave(aURL, aDocument, aDefaultFileName, aContentDisposition,
     }
 
     const kWrapColumn = 80;
-    tr.init((aChosenData ? aChosenData.uri : fileInfo.uri),
-            persistArgs.target, "", null, null, null, persist);
-    persist.progressListener = new DownloadListener(window, tr);
-    persist.saveDocument(persistArgs.source, persistArgs.target, filesFolder,
-                         persistArgs.contentType, encodingFlags, kWrapColumn);
+    persist.saveDocument(persistArgs.sourceDocument, targetFileURL, filesFolder,
+                         persistArgs.targetContentType, encodingFlags, kWrapColumn);
   } else {
-    tr.init((aChosenData ? aChosenData.uri : source),
-            persistArgs.target, "", null, null, null, persist);
-    persist.progressListener = new DownloadListener(window, tr);
-    persist.saveURI((aChosenData ? aChosenData.uri : source),
-                    null, aReferrer, persistArgs.postData, null,
-                    persistArgs.target);
+    persist.saveURI(persistArgs.sourceURI,
+                    persistArgs.sourceCacheKey, persistArgs.sourceReferrer, persistArgs.sourcePostData, null,
+                    targetFileURL);
   }
 }
 
@@ -457,134 +538,137 @@ function initFileInfo(aFI, aURL, aURLCharset, aDocument,
   }
 }
 
-Components.utils.import("resource://gre/modules/DownloadLastDir.jsm");
-
+/** 
+ * Given the Filepicker Parameters (aFpP), show the file picker dialog,
+ * prompting the user to confirm (or change) the fileName.
+ * @param aFpP
+ *        A structure (see definition in internalSave(...) method)
+ *        containing all the data used within this method.
+ * @param aSkipPrompt
+ *        If true, attempt to save the file automatically to the user's default
+ *        download directory, thus skipping the explicit prompt for a file name,
+ *        but only if the associated preference is set.
+ *        If false, don't save the file automatically to the user's
+ *        default download directory, even if the associated preference
+ *        is set, but ask for the target explicitly.
+ * @return true if the user confirmed a filename in the picker or the picker
+ *         was not displayed; false if they dismissed the picker.
+ */
 function getTargetFile(aFpP, /* optional */ aSkipPrompt)
 {
-  const prefSvcContractID = "@mozilla.org/preferences-service;1";
-  const prefSvcIID = Components.interfaces.nsIPrefService;                              
-  var prefs = Components.classes[prefSvcContractID]
-                        .getService(prefSvcIID).getBranch("browser.download.");
+  if (typeof gDownloadLastDir != "object")
+    Components.utils.import("resource://gre/modules/DownloadLastDir.jsm");
 
+  var prefs = getPrefsBrowserDownload("browser.download.");
+  var useDownloadDir = prefs.getBoolPref("useDownloadDir");
   const nsILocalFile = Components.interfaces.nsILocalFile;
 
-  var inPrivateBrowsing = false;
-  try {
-    var pbs = Components.classes["@mozilla.org/privatebrowsing;1"]
-                        .getService(Components.interfaces.nsIPrivateBrowsingService);
-    inPrivateBrowsing = pbs.privateBrowsingEnabled;
-  }
-  catch (e) {
-  }
+  if (!aSkipPrompt)
+    useDownloadDir = false;
 
-  // For information on download folder preferences, see
-  // mozilla/browser/components/preferences/main.js
-  
-  var useDownloadDir = prefs.getBoolPref("useDownloadDir");
-  var dir = null;
-  
-  // Default to lastDir if useDownloadDir is false, and lastDir
-  // is configured and valid. Otherwise, use the user's default
-  // downloads directory configured through download prefs.
-  var dnldMgr = Components.classes["@mozilla.org/download-manager;1"]
-                          .getService(Components.interfaces.nsIDownloadManager);
-  try {                          
-    var lastDir;
-    if (inPrivateBrowsing && gDownloadLastDir.file)
-      lastDir = gDownloadLastDir.file;
-    else
-      lastDir = prefs.getComplexValue("lastDir", nsILocalFile);
-    if ((!aSkipPrompt || !useDownloadDir) && lastDir.exists())
-      dir = lastDir;
-    else
-      dir = dnldMgr.userDownloadsDirectory;
-  } catch(ex) {
-    dir = dnldMgr.userDownloadsDirectory;
-  }
+  // Default to the user's default downloads directory configured
+  // through download prefs.
+  var dlMgr = Components.classes["@mozilla.org/download-manager;1"]
+                        .getService(Components.interfaces.nsIDownloadManager);
+  var dir = dlMgr.userDownloadsDirectory;
+  var dirExists = dir && dir.exists();
 
-  if (!aSkipPrompt || !useDownloadDir || !dir || (dir && !dir.exists())) {
-    if (!dir || (dir && !dir.exists())) {
-      // Default to desktop.
-      var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
-                                  .getService(Components.interfaces.nsIProperties);
-      dir = fileLocator.get("Desk", nsILocalFile);
-    }
-
-    var fp = makeFilePicker();
-    var titleKey = aFpP.fpTitleKey || "SaveLinkTitle";
-    var bundle = getStringBundle();
-    fp.init(window, bundle.GetStringFromName(titleKey), 
-            Components.interfaces.nsIFilePicker.modeSave);
-    
-    fp.defaultExtension = aFpP.fileInfo.fileExt;
-    fp.defaultString = getNormalizedLeafName(aFpP.fileInfo.fileName,
-                                             aFpP.fileInfo.fileExt);
-    appendFiltersForContentType(fp, aFpP.contentType, aFpP.fileInfo.fileExt,
-                                aFpP.saveMode);
-
-    if (dir)
-      fp.displayDirectory = dir;
-    
-    if (aFpP.isDocument) {
-      try {
-        fp.filterIndex = prefs.getIntPref("save_converter_index");
-      }
-      catch (e) {
-      }
-    }
-
-    if (fp.show() == Components.interfaces.nsIFilePicker.returnCancel || !fp.file)
-      return false;
-
-    // Do not store the last save directory as a pref inside the private browsing mode
-    var directory = fp.file.parent.QueryInterface(nsILocalFile);
-    if (inPrivateBrowsing)
-      gDownloadLastDir.file = directory;
-    else
-      prefs.setComplexValue("lastDir", nsILocalFile, directory);
-
-    fp.file.leafName = validateFileName(fp.file.leafName);
-    aFpP.saveAsType = fp.filterIndex;
-    aFpP.file = fp.file;
-    aFpP.fileURL = fp.fileURL;
-
-    if (aFpP.isDocument)
-      prefs.setIntPref("save_converter_index", aFpP.saveAsType);
-  }
-  else {
+  if (useDownloadDir && dirExists) {
     dir.append(getNormalizedLeafName(aFpP.fileInfo.fileName,
                                      aFpP.fileInfo.fileExt));
-    var file = dir;
-    
-    // Since we're automatically downloading, we don't get the file picker's 
-    // logic to check for existing files, so we need to do that here.
-    //
-    // Note - this code is identical to that in
-    //   mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in
-    // If you are updating this code, update that code too! We can't share code
-    // here since that code is called in a js component.
-    var collisionCount = 0;
-    while (file.exists()) {
-      collisionCount++;
-      if (collisionCount == 1) {
-        // Append "(2)" before the last dot in (or at the end of) the filename
-        // special case .ext.gz etc files so we don't wind up with .tar(2).gz
-        if (file.leafName.match(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i))
-          file.leafName = file.leafName.replace(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i, "(2)$&");
-        else
-          file.leafName = file.leafName.replace(/(\.[^\.]*)?$/, "(2)$&");
-      }
-      else {
-        // replace the last (n) in the filename with (n+1)
-        file.leafName = file.leafName.replace(/^(.*\()\d+\)/, "$1" + (collisionCount+1) + ")");
-      }
-    }
-    aFpP.file = file;
+    aFpP.file = uniqueFile(dir);
+    return true;
   }
 
+  // We must prompt for the file name explicitly.
+  // If we must prompt because we were asked to...
+  if (!useDownloadDir) try {
+    // ...find the directory that was last used for saving, and use it in the
+    // file picker if it is still valid. Otherwise, keep the default of the
+    // user's default downloads directory. If it doesn't exist, it will be
+    // changed to the user's desktop later.
+    var lastDir = gDownloadLastDir.file;
+    if (lastDir.exists()) {
+      dir = lastDir;
+      dirExists = true;
+    }
+  } catch(e) {}
+
+  if (!dirExists) {
+    // Default to desktop.
+    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
+                                .getService(Components.interfaces.nsIProperties);
+    dir = fileLocator.get("Desk", nsILocalFile);
+  }
+
+  var fp = makeFilePicker();
+  var titleKey = aFpP.fpTitleKey || "SaveLinkTitle";
+  fp.init(window, ContentAreaUtils.stringBundle.GetStringFromName(titleKey),
+          Components.interfaces.nsIFilePicker.modeSave);
+
+  fp.displayDirectory = dir;
+  fp.defaultExtension = aFpP.fileInfo.fileExt;
+  fp.defaultString = getNormalizedLeafName(aFpP.fileInfo.fileName,
+                                           aFpP.fileInfo.fileExt);
+  appendFiltersForContentType(fp, aFpP.contentType, aFpP.fileInfo.fileExt,
+                              aFpP.saveMode);
+
+  // The index of the selected filter is only preserved and restored if there's
+  // more than one filter in addition to "All Files".
+  if (aFpP.saveMode != SAVEMODE_FILEONLY) {
+    try {
+      fp.filterIndex = prefs.getIntPref("save_converter_index");
+    }
+    catch (e) {
+    }
+  }
+
+  if (fp.show() == Components.interfaces.nsIFilePicker.returnCancel || !fp.file)
+    return false;
+
+  if (aFpP.saveMode != SAVEMODE_FILEONLY)
+    prefs.setIntPref("save_converter_index", fp.filterIndex);
+
+  // Do not store the last save directory as a pref inside the private browsing mode
+  var directory = fp.file.parent.QueryInterface(nsILocalFile);
+  gDownloadLastDir.file = directory;
+
+  fp.file.leafName = validateFileName(fp.file.leafName);
+  
+  aFpP.saveAsType = fp.filterIndex;
+  aFpP.file = fp.file;
+  aFpP.fileURL = fp.fileURL;
   return true;
 }
 
+// Since we're automatically downloading, we don't get the file picker's
+// logic to check for existing files, so we need to do that here.
+//
+// Note - this code is identical to that in
+//   mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in
+// If you are updating this code, update that code too! We can't share code
+// here since that code is called in a js component.
+function uniqueFile(aLocalFile)
+{
+  var collisionCount = 0;
+  while (aLocalFile.exists()) {
+    collisionCount++;
+    if (collisionCount == 1) {
+      // Append "(2)" before the last dot in (or at the end of) the filename
+      // special case .ext.gz etc files so we don't wind up with .tar(2).gz
+      if (aLocalFile.leafName.match(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i))
+        aLocalFile.leafName = aLocalFile.leafName.replace(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i, "(2)$&");
+      else
+        aLocalFile.leafName = aLocalFile.leafName.replace(/(\.[^\.]*)?$/, "(2)$&");
+    }
+    else {
+      // replace the last (n) in the filename with (n+1)
+      aLocalFile.leafName = aLocalFile.leafName.replace(/^(.*\()\d+\)/, "$1" + (collisionCount + 1) + ")");
+    }
+  }
+  return aLocalFile;
+}
+
 // We have no DOM, and can only save the URL as is.
 const SAVEMODE_FILEONLY      = 0x00;
 // We have a DOM and can save as complete.
@@ -598,7 +682,6 @@ const SAVEMODE_COMPLETE_TEXT = 0x02;
 // filter must be the third filter appended.
 function appendFiltersForContentType(aFilePicker, aContentType, aFileExtension, aSaveMode)
 {
-  var bundle = getStringBundle();
   // The bundle name for saving only a specific content type.
   var bundleName;
   // The corresponding filter string for a specific content type.
@@ -654,10 +737,12 @@ function appendFiltersForContentType(aFilePicker, aContentType, aFileExtension,
   }
 
   if (aSaveMode & SAVEMODE_COMPLETE_DOM) {
-    aFilePicker.appendFilter(bundle.GetStringFromName("WebPageCompleteFilter"), filterString);
+    aFilePicker.appendFilter(ContentAreaUtils.stringBundle.GetStringFromName("WebPageCompleteFilter"),
+                             filterString);
     // We should always offer a choice to save document only if
     // we allow saving as complete.
-    aFilePicker.appendFilter(bundle.GetStringFromName(bundleName), filterString);
+    aFilePicker.appendFilter(ContentAreaUtils.stringBundle.GetStringFromName(bundleName),
+                             filterString);
   }
 
   if (aSaveMode & SAVEMODE_COMPLETE_TEXT)
@@ -670,24 +755,29 @@ function appendFiltersForContentType(aFilePicker, aContentType, aFileExtension,
 function getPostData(aDocument)
 {
   try {
-    var sessionHistory = aDocument.defaultView
-                                  .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                                  .getInterface(Components.interfaces.nsIWebNavigation)
-                                  .sessionHistory;
-    return sessionHistory.getEntryAtIndex(sessionHistory.index, false)
-                         .QueryInterface(Components.interfaces.nsISHEntry)
-                         .postData;
+    // Find the session history entry corresponding to the given document. In
+    // the current implementation, nsIWebPageDescriptor.currentDescriptor always
+    // returns a session history entry.
+    var sessionHistoryEntry =
+        aDocument.defaultView
+                 .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+                 .getInterface(Components.interfaces.nsIWebNavigation)
+                 .QueryInterface(Components.interfaces.nsIWebPageDescriptor)
+                 .currentDescriptor
+                 .QueryInterface(Components.interfaces.nsISHEntry);
+    return sessionHistoryEntry.postData;
   }
   catch (e) {
   }
   return null;
 }
 
-function getStringBundle()
+// Get the preferences branch ("browser.download." for normal 'save' mode)...
+function getPrefsBrowserDownload(branch)
 {
-  return Components.classes["@mozilla.org/intl/stringbundle;1"]
-                   .getService(Components.interfaces.nsIStringBundleService)
-                   .createBundle("chrome://global/locale/contentAreaCommands.properties");
+  const prefSvcContractID = "@mozilla.org/preferences-service;1";
+  const prefSvcIID = Components.interfaces.nsIPrefService;                              
+  return Components.classes[prefSvcContractID].getService(prefSvcIID).getBranch(branch);
 }
 
 function makeWebBrowserPersist()
@@ -706,16 +796,12 @@ function makeWebBrowserPersist()
  */
 function makeURI(aURL, aOriginCharset, aBaseURI)
 {
-  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
-                            .getService(Components.interfaces.nsIIOService);
-  return ioService.newURI(aURL, aOriginCharset, aBaseURI);
+  return ContentAreaUtils.ioService.newURI(aURL, aOriginCharset, aBaseURI);
 }
 
 function makeFileURI(aFile)
 {
-  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
-                            .getService(Components.interfaces.nsIIOService);
-  return ioService.newFileURI(aFile);
+  return ContentAreaUtils.ioService.newFileURI(aFile);
 }
 
 function makeFilePicker()
@@ -828,7 +914,7 @@ function getDefaultFileName(aDefaultFileName, aURI, aDocument,
   }
   try {
     // 7) Use the default file name
-    return getStringBundle().GetStringFromName("DefaultSaveFileName");
+    return ContentAreaUtils.stringBundle.GetStringFromName("DefaultSaveFileName");
   } catch (e) {
     //in case localized string cannot be found
   }
@@ -848,7 +934,41 @@ function validateFileName(aFileName)
   }
   else if (navigator.appVersion.indexOf("Macintosh") != -1)
     re = /[\:\/]+/g;
-  
+  else if (navigator.appVersion.indexOf("Android") != -1 ||
+           navigator.appVersion.indexOf("Maemo") != -1) {
+    // On mobile devices, the filesystem may be very limited in what
+    // it considers valid characters. To avoid errors, we sanitize
+    // conservatively.
+    const dangerousChars = "*?<>|\":/\\[];,+=";
+    var processed = "";
+    for (var i = 0; i < aFileName.length; i++)
+      processed += aFileName.charCodeAt(i) >= 32 &&
+                   !(dangerousChars.indexOf(aFileName[i]) >= 0) ? aFileName[i]
+                                                                : "_";
+
+    // Last character should not be a space
+    processed = processed.trim();
+
+    // If a large part of the filename has been sanitized, then we
+    // will use a default filename instead
+    if (processed.replace(/_/g, "").length <= processed.length/2) {
+      // We purposefully do not use a localized default filename,
+      // which we could have done using
+      // ContentAreaUtils.stringBundle.GetStringFromName("DefaultSaveFileName")
+      // since it may contain invalid characters.
+      var original = processed;
+      processed = "download";
+
+      // Preserve a suffix, if there is one
+      if (original.indexOf(".") >= 0) {
+        var suffix = original.split(".").slice(-1)[0];
+        if (suffix && suffix.indexOf("_") < 0)
+          processed += "." + suffix;
+      }
+    }
+    return processed;
+  }
+
   return aFileName.replace(re, "_");
 }
 
@@ -918,8 +1038,13 @@ function getDefaultExtension(aFilename, aURI, aContentType)
   }
 }
 
-function GetSaveModeForContentType(aContentType)
+function GetSaveModeForContentType(aContentType, aDocument)
 {
+  // We can only save a complete page if we have a loaded document
+  if (!aDocument)
+    return SAVEMODE_FILEONLY;
+
+  // Find the possible save modes using the provided content type
   var saveMode = SAVEMODE_FILEONLY;
   switch (aContentType) {
   case "text/html":
@@ -954,9 +1079,7 @@ function getCharsetforSave(aDocument)
  */
 function openURL(aURL)
 {
-  var ios = Components.classes["@mozilla.org/network/io-service;1"]
-                      .getService(Components.interfaces.nsIIOService);
-  var uri = ios.newURI(aURL, null, null);
+  var uri = makeURI(aURL);
 
   var protocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
                               .getService(Components.interfaces.nsIExternalProtocolService);
@@ -1004,7 +1127,7 @@ function openURL(aURL)
       }
     }
 
-    var channel = ios.newChannelFromURI(uri);
+    var channel = ContentAreaUtils.ioService.newChannelFromURI(uri);
     var uriLoader = Components.classes["@mozilla.org/uriloader;1"]
                               .getService(Components.interfaces.nsIURILoader);
     uriLoader.openURI(channel, true, uriListener);
