@@ -246,7 +246,8 @@ function getLocale() {
   try {
       // Get the default branch
       var defaultPrefs = gPref.getDefaultBranch(null);
-      return defaultPrefs.getCharPref(PREF_GENERAL_USERAGENT_LOCALE);
+      return defaultPrefs.getComplexValue(PREF_GENERAL_USERAGENT_LOCALE,
+                                          Ci.nsIPrefLocalizedString).data;
   } catch (e) {}
 
   return gPref.getCharPref(PREF_GENERAL_USERAGENT_LOCALE);
@@ -432,12 +433,8 @@ Blocklist.prototype = {
     var pingCountVersion = getPref("getIntPref", PREF_BLOCKLIST_PINGCOUNTVERSION, 0);
     var pingCountTotal = getPref("getIntPref", PREF_BLOCKLIST_PINGCOUNTTOTAL, 1);
     var daysSinceLastPing = 0;
-    if (pingCountVersion < 1 || pingCountTotal < 1) {
-      daysSinceLastPing = pingCountVersion == 0 ? "new" : "reset";
-      if (pingCountVersion < 1)
-        pingCountVersion = 1;
-      if (pingCountTotal < 1)
-        pingCountTotal = 1;
+    if (pingCountVersion == 0) {
+      daysSinceLastPing = "new";
     }
     else {
       // Seconds in one day is used because nsIUpdateTimerManager stores the
@@ -457,6 +454,11 @@ Blocklist.prototype = {
       }
     }
 
+    if (pingCountVersion < 1)
+      pingCountVersion = 1;
+    if (pingCountTotal < 1)
+      pingCountTotal = 1;
+
     dsURI = dsURI.replace(/%APP_ID%/g, gApp.ID);
     dsURI = dsURI.replace(/%APP_VERSION%/g, gApp.version);
     dsURI = dsURI.replace(/%PRODUCT%/g, gApp.name);
@@ -509,6 +511,7 @@ Blocklist.prototype = {
       return;
     }
 
+    LOG("Blocklist::notify: Requesting " + uri.spec);
     var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
                   createInstance(Ci.nsIXMLHttpRequest);
     request.open("GET", uri.spec, true);
@@ -857,19 +860,23 @@ Blocklist.prototype = {
         LOG("Blocklist state for " + addons[i].id + " changed from " +
             oldState + " to " + state);
 
-        // Don't warn about add-ons becoming unblocked.
-        if (state == 0)
-          continue;
-
         // We don't want to re-warn about add-ons
         if (state == oldState)
           continue;
 
+        // Ensure that softDisabled is false if the add-on is not soft blocked
+        if (state != Ci.nsIBlocklistService.STATE_SOFTBLOCKED)
+          addons[i].softDisabled = false;
+
+        // Don't warn about add-ons becoming unblocked.
+        if (state == Ci.nsIBlocklistService.STATE_NOT_BLOCKED)
+          continue;
+
         // If an add-on has dropped from hard to soft blocked just mark it as
-        // user disabled and don't warn about it.
+        // soft disabled and don't warn about it.
         if (state == Ci.nsIBlocklistService.STATE_SOFTBLOCKED &&
             oldState == Ci.nsIBlocklistService.STATE_BLOCKED) {
-          addons[i].userDisabled = true;
+          addons[i].softDisabled = true;
           continue;
         }
 
@@ -927,8 +934,10 @@ Blocklist.prototype = {
         plugins[i].blocklisted = state == Ci.nsIBlocklistService.STATE_BLOCKED;
       }
 
-      if (addonList.length == 0)
+      if (addonList.length == 0) {
+        Services.obs.notifyObservers(self, "blocklist-updated", "");
         return;
+      }
 
       if ("@mozilla.org/addons/blocklist-prompt;1" in Cc) {
         try {
@@ -938,6 +947,7 @@ Blocklist.prototype = {
         } catch (e) {
           LOG(e);
         }
+        Services.obs.notifyObservers(self, "blocklist-updated", "");
         return;
       }
 
@@ -960,11 +970,13 @@ Blocklist.prototype = {
         if (addonList[i].item instanceof Ci.nsIPluginTag)
           addonList[i].item.disabled = true;
         else
-          addonList[i].item.userDisabled = true;
+          addonList[i].item.softDisabled = true;
       }
 
       if (args.restart)
         restartApp();
+
+      Services.obs.notifyObservers(self, "blocklist-updated", "");
     });
   },
 
