@@ -62,10 +62,8 @@
 #include "nsStringEnumerator.h"
 #include "nsIServiceManager.h" 
 
-#ifdef USE_POSTSCRIPT
 #include "nsPSPrinters.h"
 #include "nsPaperPS.h"  /* Paper size list */
-#endif /* USE_POSTSCRIPT */
 
 #include "nsPrintSettingsGTK.h"
 
@@ -101,7 +99,7 @@ public:
   nsresult  InitializeGlobalPrinters();
 
   PRBool    PrintersAreAllocated()       { return mGlobalPrinterList != nsnull; }
-  PRInt32   GetNumPrinters()
+  PRUint32  GetNumPrinters()
     { return mGlobalPrinterList ? mGlobalPrinterList->Length() : 0; }
   nsString* GetStringAt(PRInt32 aInx)    { return &mGlobalPrinterList->ElementAt(aInx); }
   void      GetDefaultPrinterName(PRUnichar **aDefaultPrinterName);
@@ -478,12 +476,24 @@ NS_IMETHODIMP nsDeviceContextSpecGTK::GetSurfaceForPrinter(gfxASurface **aSurfac
       const gchar* fmtGTK = gtk_print_settings_get(mGtkPrintSettings, GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT);
       if (!fmtGTK && GTK_IS_PRINTER(mGtkPrinter)) {
         // Likely not print-to-file, check printer's capabilities
-        format = (gtk_printer_accepts_ps(mGtkPrinter)) ? nsIPrintSettings::kOutputFormatPS
-                                                       : nsIPrintSettings::kOutputFormatPDF;
-      } else if (nsDependentCString(fmtGTK).EqualsIgnoreCase("pdf")) {
-          format = nsIPrintSettings::kOutputFormatPDF;
-      } else {
+
+        // Prior to gtk 2.24, gtk_printer_accepts_pdf() and
+        // gtk_printer_accepts_ps() always returned true regardless of the
+        // printer's capability.
+        if (gtk_major_version > 2 ||
+            (gtk_major_version == 2 && gtk_minor_version >= 24)) {
+          format =
+            gtk_printer_accepts_pdf(mGtkPrinter) ?
+            static_cast<PRInt16>(nsIPrintSettings::kOutputFormatPDF) :
+            static_cast<PRInt16>(nsIPrintSettings::kOutputFormatPS);
+        } else {
           format = nsIPrintSettings::kOutputFormatPS;
+        }
+
+      } else if (nsDependentCString(fmtGTK).EqualsIgnoreCase("pdf")) {
+        format = nsIPrintSettings::kOutputFormatPDF;
+      } else {
+        format = nsIPrintSettings::kOutputFormatPS;
       }
     }
   }
@@ -548,147 +558,29 @@ NS_IMETHODIMP nsDeviceContextSpecGTK::Init(nsIWidget *aWidget,
     properPaperSize = standardGtkPaperSize;
   } else {
     properPaperSize = geckosHackishPaperSize;
-    gtk_paper_size_free(standardGtkPaperSize);
   }
   gtk_print_settings_set_paper_size(mGtkPrintSettings, properPaperSize);
   gtk_page_setup_set_paper_size_and_default_margins(mGtkPageSetup, properPaperSize);
+  gtk_paper_size_free(standardGtkPaperSize);
 
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetToPrinter(PRBool &aToPrinter)
-{
-  aToPrinter = mToPrinter;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetIsPrintPreview(PRBool &aIsPPreview)
-{
-  aIsPPreview = mIsPPreview;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetPrinterName ( const char **aPrinter )
-{
-   *aPrinter = mPrinter;
-   return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetCopies ( int &aCopies )
-{
-   aCopies = mCopies;
-   return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetFirstPageFirst(PRBool &aFpf)      
-{
-  aFpf = mFpf;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetGrayscale(PRBool &aGrayscale)      
-{
-  aGrayscale = mGrayscale;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetLandscape(PRBool &aLandscape)
-{
-  aLandscape = (mOrientation == NS_LANDSCAPE);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetTopMargin(float &aValue)      
-{
-  aValue = mTop;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetBottomMargin(float &aValue)      
-{
-  aValue = mBottom;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetRightMargin(float &aValue)      
-{
-  aValue = mRight;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetLeftMargin(float &aValue)      
-{
-  aValue = mLeft;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetCommand(const char **aCommand)      
-{
-  *aCommand = mCommand;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsDeviceContextSpecGTK::GetPath(const char **aPath)      
 {
   *aPath = mPath;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetUserCancelled(PRBool &aCancel)     
-{
-  aCancel = mCancel;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetPaperName( const char **aPaperName )
-{
-  *aPaperName = mPaperName;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetPlexName( const char **aPlexName )
-{
-  *aPlexName = mPlexName;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetResolutionName( const char **aResolutionName )
-{
-  *aResolutionName = mResolutionName;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetColorspace( const char **aColorspace )
-{
-  *aColorspace = mColorspace;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::GetDownloadFonts(PRBool &aDownloadFonts)      
-{
-  aDownloadFonts = mDownloadFonts;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsDeviceContextSpecGTK::GetPrintMethod(PrintMethod &aMethod)
 {
   return GetPrintMethod(mPrinter, aMethod);
 }
-
 /* static !! */
 nsresult nsDeviceContextSpecGTK::GetPrintMethod(const char *aPrinter, PrintMethod &aMethod)
 {
-#if defined(USE_POSTSCRIPT)
   aMethod = pmPostScript;
   return NS_OK;
-#else
-  return NS_ERROR_UNEXPECTED;
-#endif
-}
-
-NS_IMETHODIMP nsDeviceContextSpecGTK::ClosePrintManager()
-{
-  return NS_OK;
 }
 
 static void
@@ -836,14 +728,14 @@ NS_IMETHODIMP nsPrinterEnumeratorGTK::GetPrinterNameList(nsIStringEnumerator **a
     return rv;
   }
 
-  PRInt32 numPrinters = GlobalPrinters::GetInstance()->GetNumPrinters();
+  PRUint32 numPrinters = GlobalPrinters::GetInstance()->GetNumPrinters();
   nsTArray<nsString> *printers = new nsTArray<nsString>(numPrinters);
   if (!printers) {
     GlobalPrinters::GetInstance()->FreeGlobalPrinters();
     return NS_ERROR_OUT_OF_MEMORY;
   }
   
-  int count = 0;
+  PRUint32 count = 0;
   while( count < numPrinters )
   {
     printers->AppendElement(*GlobalPrinters::GetInstance()->GetStringAt(count++));
@@ -892,7 +784,6 @@ NS_IMETHODIMP nsPrinterEnumeratorGTK::InitPrintSettingsFromPrinter(const PRUnich
   if (NS_FAILED(rv))
     return rv;
 
-#ifdef USE_POSTSCRIPT
   /* "Demangle" postscript printer name */
   if (type == pmPostScript) {
     /* Strip the printing method name from the printer,
@@ -901,7 +792,6 @@ NS_IMETHODIMP nsPrinterEnumeratorGTK::InitPrintSettingsFromPrinter(const PRUnich
     if (kNotFound != slash)
       printerName.Cut(0, slash + 1);
   }
-#endif /* USE_POSTSCRIPT */
 
 #ifdef SET_PRINTER_FEATURES_VIA_PREFS
   /* Defaults to FALSE */
@@ -927,7 +817,6 @@ NS_IMETHODIMP nsPrinterEnumeratorGTK::InitPrintSettingsFromPrinter(const PRUnich
 
   aPrintSettings->SetIsInitializedFromPrinter(PR_TRUE);
 
-#ifdef USE_POSTSCRIPT
   if (type == pmPostScript) {
     DO_PR_DEBUG_LOG(("InitPrintSettingsFromPrinter() for PostScript printer\n"));
 
@@ -1064,7 +953,6 @@ NS_IMETHODIMP nsPrinterEnumeratorGTK::InitPrintSettingsFromPrinter(const PRUnich
 
     return NS_OK;    
   }
-#endif /* USE_POSTSCRIPT */
 
   return NS_ERROR_UNEXPECTED;
 }
@@ -1090,7 +978,6 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
   if (NS_FAILED(rv))
     return rv;
       
-#ifdef USE_POSTSCRIPT
   nsPSPrinterList psMgr;
   if (NS_SUCCEEDED(psMgr.Init()) && psMgr.Enabled()) {
     /* Get the list of PostScript-module printers */
@@ -1104,8 +991,7 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
       mGlobalPrinterList->AppendElement(NS_ConvertUTF8toUTF16(printerList[i]));
     }
   }
-#endif /* USE_POSTSCRIPT */  
-      
+
   /* If there are no printers available after all checks, return an error */
   if (!mGlobalPrinterList->Length())
   {
@@ -1132,7 +1018,7 @@ GlobalPrinters::GetDefaultPrinterName(PRUnichar **aDefaultPrinterName)
 {
   *aDefaultPrinterName = nsnull;
   
-  PRBool allocate = (GlobalPrinters::GetInstance()->PrintersAreAllocated() == PR_FALSE);
+  PRBool allocate = !GlobalPrinters::GetInstance()->PrintersAreAllocated();
   
   if (allocate) {
     nsresult rv = GlobalPrinters::GetInstance()->InitializeGlobalPrinters();
