@@ -53,7 +53,6 @@
 
 #include "nsCoord.h"
 #include "nsNativeThemeQt.h"
-#include "nsIDeviceContext.h"
 #include "nsPresContext.h"
 
 #include "nsRect.h"
@@ -62,7 +61,6 @@
 #include "nsThemeConstants.h"
 #include "nsILookAndFeel.h"
 #include "nsIServiceManager.h"
-#include "nsIEventStateManager.h"
 #include "nsIDOMHTMLInputElement.h"
 #include <malloc.h>
 
@@ -74,7 +72,7 @@
 #ifdef MOZ_X11
 #include "gfxXlibSurface.h"
 #endif
-#include "nsIRenderingContext.h"
+#include "nsRenderingContext.h"
 
 nsNativeThemeQt::nsNativeThemeQt()
 {
@@ -123,7 +121,7 @@ _qimage_from_gfximage_format (gfxASurface::gfxImageFormat aFormat)
 }
 
 NS_IMETHODIMP
-nsNativeThemeQt::DrawWidgetBackground(nsIRenderingContext* aContext,
+nsNativeThemeQt::DrawWidgetBackground(nsRenderingContext* aContext,
                                       nsIFrame* aFrame,
                                       PRUint8 aWidgetType,
                                       const nsRect& aRect,
@@ -172,7 +170,7 @@ nsNativeThemeQt::DrawWidgetBackground(nsIRenderingContext* aContext,
 
 nsresult
 nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
-                                      nsIRenderingContext* aContext,
+                                      nsRenderingContext* aContext,
                                       nsIFrame* aFrame,
                                       PRUint8 aWidgetType,
                                       const nsRect& aRect,
@@ -200,7 +198,7 @@ nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
     QMatrix qctm(ctm.xx, ctm.yx, ctm.xy, ctm.yy, ctm.x0, ctm.y0);
     qPainter->setWorldMatrix(qctm, true);
 
-    PRInt32 p2a = GetAppUnitsPerDevPixel(aContext);
+    PRInt32 p2a = aContext->AppUnitsPerDevPixel();
 
     QRect r = qRectInPixels(aRect, p2a);
     QRect cr = qRectInPixels(aClipRect, p2a);
@@ -347,7 +345,7 @@ nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
 }
 
 NS_IMETHODIMP
-nsNativeThemeQt::GetWidgetBorder(nsIDeviceContext* aContext,
+nsNativeThemeQt::GetWidgetBorder(nsDeviceContext* ,
                                  nsIFrame* aFrame,
                                  PRUint8 aWidgetType,
                                  nsIntMargin* aResult)
@@ -370,7 +368,7 @@ nsNativeThemeQt::GetWidgetBorder(nsIDeviceContext* aContext,
 }
 
 PRBool
-nsNativeThemeQt::GetWidgetPadding(nsIDeviceContext* ,
+nsNativeThemeQt::GetWidgetPadding(nsDeviceContext* ,
                                   nsIFrame*, PRUint8 aWidgetType,
                                   nsIntMargin* aResult)
 {
@@ -386,7 +384,7 @@ nsNativeThemeQt::GetWidgetPadding(nsIDeviceContext* ,
 }
 
 NS_IMETHODIMP
-nsNativeThemeQt::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame* aFrame,
+nsNativeThemeQt::GetMinimumWidgetSize(nsRenderingContext* aContext, nsIFrame* aFrame,
                                       PRUint8 aWidgetType,
                                       nsIntSize* aResult, PRBool* aIsOverridable)
 {
@@ -395,7 +393,7 @@ nsNativeThemeQt::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame* a
 
     QStyle *s = qApp->style();
 
-    PRInt32 p2a = GetAppUnitsPerDevPixel(aContext);
+    PRInt32 p2a = aContext->AppUnitsPerDevPixel();
 
     switch (aWidgetType) {
     case NS_THEME_RADIO:
